using System.Data;
using NHibernate.Cfg;
using NHibernate.Dialect.Function;
using NHibernate.SqlCommand;
using NHibernate.Util;
using System.Text;

namespace NHibernate.Dialect
{
    public class Cache71Dialect : Dialect
    {

        public Cache71Dialect()
            : base()
        {
            CommonRegistration();
            Register71Functions();
        }

        protected void CommonRegistration()
        {
            RegisterColumnType(DbType.AnsiString, "VARCHAR($1)");
            RegisterColumnType(DbType.AnsiStringFixedLength, "VARCHAR($1)");
            RegisterColumnType(DbType.Binary, "VARBINARY($1)");
            RegisterColumnType(DbType.Boolean, "BIT");
            RegisterColumnType(DbType.Byte, "INT");
            RegisterColumnType(DbType.Currency, "MONEY");
            RegisterColumnType(DbType.Date, "DATE");
            RegisterColumnType(DbType.DateTime, "DATETIME");
            RegisterColumnType(DbType.Decimal, "DEC($1)");
            RegisterColumnType(DbType.Double, "DOUBLE");
            RegisterColumnType(DbType.Guid, "VARCHAR(255)");
            RegisterColumnType(DbType.Int16, "SMALLINT");
            RegisterColumnType(DbType.Int32, "INTEGER");
            RegisterColumnType(DbType.Int64, "BIGINT");
            RegisterColumnType(DbType.Single, "FLOAT($1)");
            RegisterColumnType(DbType.String, "VARCHAR($1)");
            RegisterColumnType(DbType.StringFixedLength, "CHAR($1)");
            RegisterColumnType(DbType.Time, "TIME");
            RegisterColumnType(DbType.VarNumeric, "NUMBERIC($1, $2)");

            RegisterFunction("abs", new StandardSQLFunction("abs"));
            RegisterFunction("acos", new StandardSQLFunction("acos"));
            RegisterFunction("%alphaup", new StandardSQLFunction("%alphaup", NHibernateUtil.String));
            RegisterFunction("ascii", new StandardSQLFunction("ascii", NHibernateUtil.String));
            RegisterFunction("asin", new StandardSQLFunction("asin", NHibernateUtil.Double));
            RegisterFunction("atan", new StandardSQLFunction("atan", NHibernateUtil.Double));
            RegisterFunction("bit_length", new SQLFunctionTemplate(NHibernateUtil.Int32, "($length(?1)*8)"));
            // hibernate impelemnts cast in Dialect.java
            RegisterFunction("ceiling", new StandardSQLFunction("ceiling", NHibernateUtil.Int32));
            RegisterFunction("char", new StandardSQLFunction("char", NHibernateUtil.Character));
            RegisterFunction("character_length", new StandardSQLFunction("character_length", NHibernateUtil.Int32));
            RegisterFunction("char_length", new StandardSQLFunction("char_length", NHibernateUtil.Int32));
            RegisterFunction("cos", new StandardSQLFunction("cos", NHibernateUtil.Double));
            RegisterFunction("cot", new StandardSQLFunction("cot", NHibernateUtil.Double));
            RegisterFunction("coalesce", new VarArgsSQLFunction("coalesce(", ",", ")"));
            RegisterFunction("concat", new VarArgsSQLFunction(NHibernateUtil.String, "", "||", ""));
            RegisterFunction("curdate", new StandardSQLFunction("curdate", NHibernateUtil.Date));
            RegisterFunction("current_date", new NoArgSQLFunction("current_date", NHibernateUtil.Date, false));
            RegisterFunction("current_time", new NoArgSQLFunction("current_time", NHibernateUtil.Time, false));
            RegisterFunction("curtime", new StandardSQLFunction("curtime", NHibernateUtil.Time));
            RegisterFunction("database", new StandardSQLFunction("database", NHibernateUtil.String));
            RegisterFunction("dateadd", new VarArgsSQLFunction(NHibernateUtil.Timestamp, "dateadd(", ",", ")"));
            RegisterFunction("datediff", new VarArgsSQLFunction(NHibernateUtil.Int32, "datediff(", ",", ")"));
            RegisterFunction("datename", new VarArgsSQLFunction(NHibernateUtil.String, "datename(", ",", ")"));
            RegisterFunction("datepart", new VarArgsSQLFunction(NHibernateUtil.Int32, "datepart(", ",", ")"));
            RegisterFunction("day", new StandardSQLFunction("day", NHibernateUtil.Int32));
            RegisterFunction("dayname", new StandardSQLFunction("dayname", NHibernateUtil.String));
            RegisterFunction("dayofmonth", new StandardSQLFunction("dayofmonth", NHibernateUtil.Int32));
            RegisterFunction("dayofweek", new StandardSQLFunction("dayofweek", NHibernateUtil.Int32));
            RegisterFunction("dayofyear", new StandardSQLFunction("dayofyear", NHibernateUtil.Int32));
            // is it necessary to register %exact since it can only appear in a where clause?
            RegisterFunction("%exact", new StandardSQLFunction("%exact", NHibernateUtil.String));
            RegisterFunction("exp", new StandardSQLFunction("exp", NHibernateUtil.Double));
            RegisterFunction("%external", new StandardSQLFunction("%external", NHibernateUtil.String));
            RegisterFunction("$extract", new VarArgsSQLFunction(NHibernateUtil.Int32, "$extract(", ",", ")"));
            RegisterFunction("$find", new VarArgsSQLFunction(NHibernateUtil.Int32, "$find(", ",", ")"));
            RegisterFunction("floor", new StandardSQLFunction("floor", NHibernateUtil.Int32));
            RegisterFunction("getdate", new StandardSQLFunction("getdate", NHibernateUtil.Timestamp));
            RegisterFunction("hour", new StandardSQLFunction("hour", NHibernateUtil.Int32));
            RegisterFunction("ifnull", new VarArgsSQLFunction("ifnull(", ",", ")"));
            RegisterFunction("%internal", new StandardSQLFunction("%internal"));
            RegisterFunction("isnull", new VarArgsSQLFunction("isnull(", ",", ")"));
            RegisterFunction("isnumeric", new StandardSQLFunction("isnumeric", NHibernateUtil.Int32));
            RegisterFunction("lcase", new StandardSQLFunction("lcase", NHibernateUtil.String));
            RegisterFunction("left", new StandardSQLFunction("left", NHibernateUtil.String));
            RegisterFunction("len", new StandardSQLFunction("len", NHibernateUtil.Int32));
            RegisterFunction("length", new StandardSQLFunction("length", NHibernateUtil.Int32));
            RegisterFunction("$length", new VarArgsSQLFunction("$length(", ",", ")"));
            // aggregate functions shouldn't be registered, right?
            //RegisterFunction( "list", new StandardSQLFunction("list",NHibernateUtil.STRING) );
            // stopped on $list
            RegisterFunction("$list", new VarArgsSQLFunction("$list(", ",", ")"));
            RegisterFunction("$listdata", new VarArgsSQLFunction("$listdata(", ",", ")"));
            RegisterFunction("$listfind", new VarArgsSQLFunction("$listfind(", ",", ")"));
            RegisterFunction("$listget", new VarArgsSQLFunction("$listget(", ",", ")"));
            RegisterFunction("$listlength", new StandardSQLFunction("$listlength", NHibernateUtil.Int32));
            RegisterFunction("locate", new StandardSQLFunction("$FIND", NHibernateUtil.Int32));
            RegisterFunction("log", new StandardSQLFunction("log", NHibernateUtil.Double));
            RegisterFunction("log10", new StandardSQLFunction("log", NHibernateUtil.Double));
            RegisterFunction("lower", new StandardSQLFunction("lower"));
            RegisterFunction("ltrim", new StandardSQLFunction("ltrim"));
            RegisterFunction("minute", new StandardSQLFunction("minute", NHibernateUtil.Int32));
            RegisterFunction("mod", new StandardSQLFunction("mod", NHibernateUtil.Double));
            RegisterFunction("month", new StandardSQLFunction("month", NHibernateUtil.Int32));
            RegisterFunction("monthname", new StandardSQLFunction("monthname", NHibernateUtil.String));
            RegisterFunction("now", new StandardSQLFunction("monthname", NHibernateUtil.Timestamp));
            RegisterFunction("nullif", new VarArgsSQLFunction("nullif(", ",", ")"));
            RegisterFunction("nvl", new NvlFunction());
            RegisterFunction("%odbcin", new StandardSQLFunction("%odbcin"));
            RegisterFunction("%odbcout", new StandardSQLFunction("%odbcin"));
            RegisterFunction("%pattern", new VarArgsSQLFunction(NHibernateUtil.String, "", "%pattern", ""));
            RegisterFunction("pi", new StandardSQLFunction("pi", NHibernateUtil.Double));
            RegisterFunction("$piece", new VarArgsSQLFunction(NHibernateUtil.String, "$piece(", ",", ")"));
            RegisterFunction("position", new VarArgsSQLFunction(NHibernateUtil.Int32, "position(", " in ", ")"));
            RegisterFunction("power", new VarArgsSQLFunction(NHibernateUtil.String, "power(", ",", ")"));
            RegisterFunction("quarter", new StandardSQLFunction("quarter", NHibernateUtil.Int32));
            RegisterFunction("repeat", new VarArgsSQLFunction(NHibernateUtil.String, "repeat(", ",", ")"));
            RegisterFunction("replicate", new VarArgsSQLFunction(NHibernateUtil.String, "replicate(", ",", ")"));
            RegisterFunction("right", new StandardSQLFunction("right", NHibernateUtil.String));
            RegisterFunction("round", new VarArgsSQLFunction(NHibernateUtil.Single, "round(", ",", ")"));
            RegisterFunction("rtrim", new StandardSQLFunction("rtrim", NHibernateUtil.String));
            RegisterFunction("second", new StandardSQLFunction("second", NHibernateUtil.Int32));
            RegisterFunction("sign", new StandardSQLFunction("sign", NHibernateUtil.Int32));
            RegisterFunction("sin", new StandardSQLFunction("sin", NHibernateUtil.Double));
            RegisterFunction("space", new StandardSQLFunction("space", NHibernateUtil.String));
            RegisterFunction("%sqlstring", new VarArgsSQLFunction(NHibernateUtil.String, "%sqlstring(", ",", ")"));
            RegisterFunction("%sqlupper", new VarArgsSQLFunction(NHibernateUtil.String, "%sqlupper(", ",", ")"));
            RegisterFunction("sqrt", new StandardSQLFunction("SQRT", NHibernateUtil.Double));
            RegisterFunction("%startswith", new VarArgsSQLFunction(NHibernateUtil.String, "", "%startswith", ""));
            // below is for Cache' that don't have str in 2007.1 there is str and we register str directly
            RegisterFunction("str", new SQLFunctionTemplate(NHibernateUtil.String, "cast(?1 as char varying)"));
            RegisterFunction("string", new VarArgsSQLFunction(NHibernateUtil.String, "string(", ",", ")"));
            // note that %string is deprecated
            RegisterFunction("%string", new VarArgsSQLFunction(NHibernateUtil.String, "%string(", ",", ")"));
            RegisterFunction("substr", new VarArgsSQLFunction(NHibernateUtil.String, "substr(", ",", ")"));
            RegisterFunction("substring", new VarArgsSQLFunction(NHibernateUtil.String, "substring(", ",", ")"));
            RegisterFunction("sysdate", new NoArgSQLFunction("sysdate", NHibernateUtil.Time, false));
            RegisterFunction("tan", new StandardSQLFunction("tan", NHibernateUtil.Double));
            RegisterFunction("timestampadd", new StandardSQLFunction("timestampadd", NHibernateUtil.Double));
            RegisterFunction("timestampdiff", new StandardSQLFunction("timestampdiff", NHibernateUtil.Double));
            RegisterFunction("tochar", new VarArgsSQLFunction(NHibernateUtil.String, "tochar(", ",", ")"));
            RegisterFunction("to_char", new VarArgsSQLFunction(NHibernateUtil.String, "to_char(", ",", ")"));
            RegisterFunction("todate", new VarArgsSQLFunction(NHibernateUtil.String, "todate(", ",", ")"));
            RegisterFunction("to_date", new VarArgsSQLFunction(NHibernateUtil.String, "todate(", ",", ")"));
            RegisterFunction("tonumber", new StandardSQLFunction("tonumber"));
            RegisterFunction("to_number", new StandardSQLFunction("tonumber"));
            // TRIM(end_keyword string-expression-1 FROM string-expression-2)
            // use Hibernate implementation "From" is one of the parameters they pass in position ?3
            //RegisterFunction( "trim", new SQLFunctionTemplate(Hibernate.STRING, "trim(?1 ?2 from ?3)") );
            RegisterFunction("truncate", new StandardSQLFunction("truncate", NHibernateUtil.String));
            RegisterFunction("ucase", new StandardSQLFunction("ucase", NHibernateUtil.String));
            RegisterFunction("upper", new StandardSQLFunction("upper"));
            // %upper is deprecated
            RegisterFunction("%upper", new StandardSQLFunction("%upper"));
            RegisterFunction("user", new StandardSQLFunction("user", NHibernateUtil.String));
            RegisterFunction("week", new StandardSQLFunction("user", NHibernateUtil.Int32));
            RegisterFunction("xmlconcat", new VarArgsSQLFunction(NHibernateUtil.String, "xmlconcat(", ",", ")"));
            RegisterFunction("xmlelement", new VarArgsSQLFunction(NHibernateUtil.String, "xmlelement(", ",", ")"));
            // xmlforest requires a new kind of function constructor
            RegisterFunction("year", new StandardSQLFunction("year", NHibernateUtil.Int32));

            DefaultProperties[Environment.ConnectionDriver] = "NHibernate.Driver.CacheDriver";
        }

        protected void Register71Functions()
        {
            RegisterFunction("str", new VarArgsSQLFunction(NHibernateUtil.String, "str(", ",", ")"));
        }

        public override bool HasAlterTable
        {
            get
            {
                return true;
            }
        }

        public override bool QualifyIndexName
        {
            get
            {
                return false;
            }
        }

        public override bool SupportsUnique
        {
            get
            {
                return true;
            }
        }

        public override string GetAddForeignKeyConstraintString(string constraintName, string[] foreignKey, string referencedTable, string[] primaryKey)
        {
            return new StringBuilder(300)
                .Append(" ADD CONSTRAINT ")
                .Append(constraintName)
                .Append(" FOREIGN KEY ")
                .Append(constraintName)
                .Append(" (")
                .Append(StringHelper.Join(", ", foreignKey))
                .Append(") REFERENCES ")
                .Append(referencedTable)
                .Append(" (")
                .Append(StringHelper.Join(", ", primaryKey))
                .Append(") ")
                .ToString();
        }

        public override bool SupportsColumnCheck()
        {
            return false;
        }

        public override string AddColumnString
        {
            get
            {
                return "add column";
            }
        }

        protected override string CascadeConstraintsString
        {
            get
            {
                return "";
            }
        }

        public override bool DropConstraints
        {
            get
            {
                return true;
            }
        }

        public override bool SupportsCommentOn
        {
            get
            {
                return false;
            }
        }

        public override bool SupportsIdentityColumns
        {
            get
            {
                return true;
            }
        }

        public override bool HasDataTypeInIdentityColumn
        {
            get
            {
                return true;
            }
        }

        public override string GetIdentitySelectString(string identityColumn, string tableName)
        {
            return "SELECT LAST_IDENTITY() FROM %TSQL_sys.snf";
        }

        public override bool SupportsSequences
        {
            get
            {
                return false;
            }
        }

        public override bool SupportsLimit
        {
            get
            {
                return true;
            }
        }

        public override bool SupportsLimitOffset
        {
            get
            {
                return false;
            }
        }

        public override bool SupportsVariableLimit
        {
            get
            {
                return true;
            }
        }

        public override bool BindLimitParametersFirst
        {
            get
            {
                return true;
            }
        }

        public override bool UseMaxForLimit
        {
            get
            {
                return true;
            }
        }

        public override SqlString GetLimitString(SqlString querySqlString, bool hasOffset)
        {
            if (hasOffset)
            {
                throw new System.NotSupportedException("An offset may not be specified to <TOP n> in Cache SQL");
            }

            int insertionPoint = querySqlString.StartsWithCaseInsensitive("select distinct") ? 15 : 6;
            return new SqlStringBuilder(querySqlString.Length + 8)
                        .Add(querySqlString)
                        .Insert(insertionPoint, " TOP ? ")
                        .ToSqlString();
        }

        public override string IdentityColumnString
        {
            get
            {
                return "identity";
            }
        }

        public override string LowercaseFunction
        {
            get
            {
                return "lower";
            }
        }

        public override string NullColumnString
        {
            get
            {
                return " null";
            }
        }

        public override JoinFragment CreateOuterJoinFragment()
        {
            return new CacheJoinFragment();
        }

        public override string NoColumnsInsertString
        {
            get
            {
                return " default values";
            }
        }

    }

}
