using System;
using System.Data;
using NHibernate.SqlCommand;

namespace NHibernate.Dialect
{
	/// <summary>
	/// A SQL dialect for SQLite.
	/// </summary>
	/// <remarks>
	/// <p>
	/// Author: <a href="mailto:ib@stalker.ro"> Ioan Bizau </a>
	/// </p>
	/// </remarks>
	public class SQLiteDialect : Dialect
	{
		/// <summary>
		/// 
		/// </summary>
		public SQLiteDialect()
		{
			RegisterColumnType(DbType.Binary, "BLOB");
			RegisterColumnType(DbType.Byte, "INTEGER");
			RegisterColumnType(DbType.Int16, "INTEGER");
			RegisterColumnType(DbType.Int32, "INTEGER");
			RegisterColumnType(DbType.Int64, "INTEGER");
			RegisterColumnType(DbType.SByte, "INTEGER");
			RegisterColumnType(DbType.UInt16, "INTEGER");
			RegisterColumnType(DbType.UInt32, "INTEGER");
			RegisterColumnType(DbType.UInt64, "INTEGER");
			RegisterColumnType(DbType.Currency, "NUMERIC");
			RegisterColumnType(DbType.Decimal, "NUMERIC");
			RegisterColumnType(DbType.Double, "NUMERIC");
			RegisterColumnType(DbType.Single, "NUMERIC");
			RegisterColumnType(DbType.VarNumeric, "NUMERIC");
			RegisterColumnType(DbType.String, "TEXT");
			RegisterColumnType(DbType.AnsiStringFixedLength, "TEXT");
			RegisterColumnType(DbType.StringFixedLength, "TEXT");

			RegisterColumnType(DbType.DateTime, "DATETIME");
			RegisterColumnType(DbType.Time, "DATETIME");
			RegisterColumnType(DbType.Boolean, "INTEGER");
			RegisterColumnType(DbType.Guid, "UNIQUEIDENTIFIER");
		}

		/// <summary>
		/// 
		/// </summary>
		public override string GetIdentitySelectString(string identityColumn, string tableName)
		{
			return "select last_insert_rowid()";
		}

		public override SqlString AddIdentitySelectToInsert(SqlString insertSql, string identityColumn, string tableName)
		{
			return insertSql.Append("; " + GetIdentitySelectString(identityColumn, tableName));
		}

        /*
         * Not supported in this version (copy from rc)
        public override bool SupportsInsertSelectIdentity
        {
            get { return true; }
        }
        */

		public override bool HasAlterTable
		{
			get { return true; }
		}

		public override bool DropConstraints
		{
			get { return false; }
		}

		public override string ForUpdateString
		{
			get { return string.Empty; }
		}
		
		public override bool SupportsSubSelects
		{
			get { return true; }
		}

        public override bool SupportsIfExistsBeforeTableName
        {
            get { return true; }
        }

        public override bool HasDataTypeInIdentityColumn
        {
            get { return false; }
        }

        public override bool SupportsIdentityColumns
        {
            get { return true; }
        }

        public override bool SupportsLimit
        {
            get { return true; }
        }

        public override string IdentityColumnString
        {
            get { return "integer"; }
        }

        /*
         * Not supported in this version (copy from rc)
        public override string Qualify(string catalog, string schema, string table)
        {
            StringBuilder qualifiedName = new StringBuilder();

            if (!string.IsNullOrEmpty(catalog))
            {
                    if (catalog.EndsWith(CloseQuote.ToString()))
                        catalog = catalog.Substring(0, catalog.Length - 1);
                    qualifiedName.Append(catalog).Append(StringHelper.Underscore);
            }
            if (!string.IsNullOrEmpty(schema))
            {
                if (schema.StartsWith(OpenQuote.ToString()))
                    schema = schema.Substring(0, schema.Length-1);
                qualifiedName.Append(schema).Append(StringHelper.Underscore);
            }

            if (table.StartsWith(OpenQuote.ToString()))
                table = table.Substring(1, table.Length - 1);

            return qualifiedName.Append(table).ToString();
        }
        */

        public override string NoColumnsInsertString
        {
            get { return "DEFAULT VALUES"; }
        }

        public override SqlString GetLimitString(SqlString querySqlString, int offset, int limit)
        {
            SqlStringBuilder pagingBuilder = new SqlStringBuilder();
            pagingBuilder.Add(querySqlString);
            pagingBuilder.Add(" limit ");
            pagingBuilder.Add(limit.ToString());

            if (offset > 0)
            {
                pagingBuilder.Add(", ");
                pagingBuilder.Add(offset.ToString());
            }

            return pagingBuilder.ToSqlString();            
        }

        public override string GetAddForeignKeyConstraintString(string constraintName, string[] foreignKey, string referencedTable, string[] primaryKey)
        {
            return null;
        }

        public override string GetDropForeignKeyConstraintString(string constraintName)
        {
            return null;
        }

        public override string GetAddPrimaryKeyConstraintString(string constraintName)
        {
            return null;
        }
        
	}
}