using System;
using System.Collections;

namespace ERModel
{
	/// <summary>
	/// Summary description for ERColumn.
	/// </summary>
	public class ERColumn : ERElement
	{
		private bool _primaryKey = false;
		private bool _foreignKey = false;
		private string _referencedColumn;
		private int _type;
		private int _length;
		private bool _nullable;
		private string _idGenerator;
		private string _idGeneratorKey;
		private bool _unique = false;
		private string _checkConstraint = null;
		private string _defaultValue = null;
		private int _scale;
		private readonly EREntity _entity;
		private string _comment;
		private Hashtable _userTypes;

		public ERColumn(EREntity entity):base(entity.GetModel())
		{
			_entity = entity;
			_entity.AddElement(this);
			_userTypes = new Hashtable();
		}

		public EREntity GetEntity()
		{
			return _entity;
		}

		public bool ForeignKey
		{
			get { return _foreignKey; }
			set {_foreignKey = value; }
		}

		public string ReferencedColumn
		{
			get {return _referencedColumn;}
			set {_referencedColumn = value;}
		}

		public int Length
		{
			get { return _length; }
			set { _length = value; }
		}
		
		public bool Nullable
		{
			get { return _nullable; }
			set { _nullable = value; }
		}

		public bool PrimaryKey 
		{
			get {return _primaryKey;}
			set {_primaryKey = value; }
		}

		public int Type
		{
			get { return _type; }
			set { _type = value; }
		}
		
		public string IdGenerator
		{
			get {return _idGenerator;}
			set {_idGenerator = value;}
		}

		public string IdGeneratorKey
		{
			get {return _idGeneratorKey;}
			set {_idGeneratorKey = value; }
		}

		public string CheckConstraint
		{
			get {return _checkConstraint;}
			set { _checkConstraint = value;}
		}

		public string DefaultValue
		{
			get {return _defaultValue;}
			set {_defaultValue = value;}
		}

		public bool HasCheckConstraint()
		{
			return _checkConstraint != null && _checkConstraint.Length > 0;
		}

		public bool Unique
		{
			get {return _unique;}
			set {_unique = value; }
		}

		public int Scale
		{
			get {return _scale;}
			set {_scale = value;}
		}

		public string Comment
		{
			get{return _comment;}
			set{_comment = value;}
		}

		public void AddUserType(int aDBType, string aType) 
		{
			_userTypes.Add(aDBType, aType);
		}
    
		public string GetUserType(int aDBType) 
		{
			return (string) _userTypes[aDBType];
		}
	}
}
