using System;

namespace ERModel
{
	/// <summary>
	/// Summary description for EREntity.
	/// </summary>
	public class EREntity : ERElement
	{
		private System.Collections.IList _elements;
		private System.Collections.IDictionary _uniqueKeys;
//		private System.Collections.IList _checkConstraints;
		private System.Collections.IDictionary _checkContraints;
		private System.Collections.IDictionary _indexs;
		private string _pkConstraintName;
		private string _comment;
		private string _schema;
		private string _extraDDLClause;

		public EREntity(ERModel model):base(model)
		{
			_elements = new System.Collections.ArrayList();
			_uniqueKeys = new System.Collections.Hashtable();
			_indexs = new System.Collections.Hashtable();
//			_checkConstraints = new System.Collections.ArrayList();
			_checkContraints = new System.Collections.Hashtable();
		}

		public void AddElement(ERElement element)
		{
			_elements.Add(element);
		}

		public System.Collections.IList Elements
		{
			get {return _elements; }
		}

		public System.Collections.IEnumerator GetElementsEnumerator()
		{
			return _elements.GetEnumerator();
		}

		public ERColumn GetColumnByName(String columnName)
		{
			ERColumn result = null;
			foreach(ERElement element in _elements)
			{
				if(element is ERColumn)
				if(element.GetName().Equals(columnName))
				{
					result = element as ERColumn ;
					break;
				}
			}
			return result;
		}

		public System.Collections.IEnumerator uniqueKeysEnumerator()
		{
			return _uniqueKeys.Keys.GetEnumerator();
		}

		public System.Collections.IEnumerator UniqueKeyColumnsEnumerator(string uniqueKey)
		{
			object obj = _uniqueKeys[uniqueKey];
			if(obj != null)
			{
				return ((System.Collections.IList)obj).GetEnumerator();
			}
			return null;
		}

		public void addUniqueKey(string uniqueKey, string column)
		{
			System.Collections.IList list = _uniqueKeys[uniqueKey] as System.Collections.IList;
			if(list == null)
			{
				list = new System.Collections.ArrayList();
				_uniqueKeys.Add(uniqueKey, list);
			}
			list.Add(column);
		}
		
		public System.Collections.IEnumerator CheckConstraintKeysEnumerator()
		{
			return _checkContraints.Keys.GetEnumerator();
		}
		
		public void AddCheckConstraint(string name, string constraint)
		{
			_checkContraints.Add(name, constraint);
		}

		public string GetCheckConstraints(string constraint)
		{
			return (string)_checkContraints[constraint];
		}

		//add index
		public void addIndex(string indexKey, string column)
		{
			System.Collections.IList list = _indexs[indexKey] as System.Collections.IList;
			if(list == null)
			{
				list = new System.Collections.ArrayList();
				_indexs.Add(indexKey, list);
			}
			list.Add(column);
		}

		public System.Collections.IEnumerator indexKeysEnumerator()
		{
			return _indexs.Keys.GetEnumerator();
		}
		
		public System.Collections.IEnumerator indexKeyColumnsEnumerator(string indexKey)
		{
			object obj = _indexs[indexKey];
			if(obj != null)
			{
				return ((System.Collections.IList)obj).GetEnumerator();
			}
			return null;
		}
		
		public string PKConstraintName
		{
			get{return _pkConstraintName;}
			set{_pkConstraintName = value;}
		}

		public string Comment
		{
			get{return _comment;}
			set{_comment = value;}
		}

		public string Schema
		{
			get{
				if (_schema != null && _schema.Length == 0) 
				{
					return null;
				}
				return _schema;
			}
			set{_schema = value;}
		}

		public string ExtraDDLClause
		{
			get
			{
				if (_extraDDLClause != null && _extraDDLClause.Length == 0) 
				{
					return null;
				}
				return _extraDDLClause;
			}
			set{_extraDDLClause = value;}
		}
	}
}
