using System;
using System.Xml;
using ERModel;

namespace ERModel.Handler
{
	/// <summary>
	/// Summary description for ERCloumnHandler.
	/// </summary>
	public class ERColumnHandler
	{
		public ERColumn ReadERElement(EREntity entity, XmlNode xmlNode)
		{
			ERColumn column = new ERColumn(entity);
			column.SetName(xmlNode.Attributes["name"].Value);
			foreach(XmlNode childNode in xmlNode.ChildNodes)
			{
				if(childNode.Name.Equals("PrimaryKey"))
				{
					column.PrimaryKey = ERElementHandlerUtil.GetTextAsBoolean(childNode);
				}
				else if(childNode.Name.Equals("ForeignKey"))
				{
					column.ForeignKey = ERElementHandlerUtil.GetTextAsBoolean(childNode);
				}
				else if(childNode.Name.Equals("ReferencedColumn"))
				{
					column.ReferencedColumn = childNode.InnerText;
				}
				else if(childNode.Name.Equals("Type"))
				{
					column.Type = ERElementHandlerUtil.GetTextAsInt(childNode);
				}
				else if(childNode.Name.Equals("Length"))
				{
					column.Length = ERElementHandlerUtil.GetTextAsInt(childNode);
				}
				else if(childNode.Name.Equals("Nullable"))
				{
					column.Nullable = ERElementHandlerUtil.GetTextAsBoolean(childNode);
				}
				else if(childNode.Name.Equals("IDGenerator"))
				{
					column.IdGenerator = childNode.InnerText;
				}
				else if(childNode.Name.Equals("IDGeneratorKey"))
				{
					column.IdGeneratorKey = childNode.InnerText;
				}
				else if(childNode.Name.Equals("Unique"))
				{
					column.Unique = ERElementHandlerUtil.GetTextAsBoolean(childNode);
				}
				else if(childNode.Name.Equals("CheckConstraint"))
				{
					column.CheckConstraint = xmlNode.InnerText;
				}
				else if(childNode.Name.Equals("DefaultValue"))
				{
					column.DefaultValue = childNode.InnerText;
				}
				else if(childNode.Name.Equals("Comment"))
				{
					column.Comment = childNode.InnerText;
				}
				else if(childNode.Name.Equals("Scale"))
				{
					column.Scale = ERElementHandlerUtil.GetTextAsInt(childNode);
				}
				else if(childNode.Name.Equals("UserTypes"))
				{
					foreach(XmlNode userTypeNode in childNode.ChildNodes) 
					{
						if (userTypeNode.Name.Equals("UserType")) 
						{
							string lDBType = userTypeNode.Attributes["DBType"].Value;
							string lType = userTypeNode.InnerText;
							column.AddUserType(int.Parse(lDBType), lType);
						}
					}
				}
			}
			return column;
		}
	}
}
