using System;
using System.Xml;
using ERModel.Handler;

namespace ERModel.Handler
{
	/// <summary>
	/// Summary description for EREntityHandler.
	/// </summary>
	public class EREntityHandler
	{
		private ERColumnHandler _columnHandler;
		public EREntityHandler()
		{
			_columnHandler = new ERColumnHandler();
		}

		public EREntity ReadERElement(ERModel model, XmlNode xmlNode)
		{
			EREntity entity = new EREntity(model);
			entity.SetName(xmlNode.Attributes["name"].Value);
			System.Xml.XmlAttribute lPKNameAttribute =  xmlNode.Attributes["PKConstraintName"];
			if(lPKNameAttribute != null)
			{
				entity.PKConstraintName =lPKNameAttribute.Value;
			}
			System.Xml.XmlAttribute lSchemaAttribute =  xmlNode.Attributes["schema"];
			if(lSchemaAttribute != null)
			{
				entity.Schema =lSchemaAttribute.Value;
			}

			XmlNodeList columnNodeList = xmlNode.SelectNodes("Column");
			foreach(XmlNode cloumnNode in columnNodeList)
			{
				_columnHandler.ReadERElement(entity, cloumnNode);
			}

			XmlNodeList uniqueKeyNodeList = xmlNode.SelectNodes("UniqueKey");
			foreach(XmlNode uniqueKeyNode in uniqueKeyNodeList)
			{
				string uniqueKey = uniqueKeyNode.Attributes["name"].Value;
				XmlNodeList uniqueKeyColumnNodeList = uniqueKeyNode.SelectNodes("Column");
				foreach(XmlNode columnNode in uniqueKeyColumnNodeList)
				{
					entity.addUniqueKey(uniqueKey, columnNode.Attributes["name"].Value);
				}
			}

			XmlNodeList checkConstraintNodeList = xmlNode.SelectNodes("CheckConstraint");
			foreach(XmlNode checkConstraintNode in checkConstraintNodeList)
			{
				string lName = checkConstraintNode.Attributes["name"].Value;
				XmlNodeList constraintColumnNodeList = checkConstraintNode.SelectNodes("Column");
				foreach(XmlNode columnNode in constraintColumnNodeList)
				{
					entity.AddCheckConstraint(lName, columnNode.Attributes["name"].Value);
				}
			}

			XmlNodeList indexNodeList = xmlNode.SelectNodes("Index");
			foreach (XmlNode indexNode in indexNodeList)
			{	
				string indexKey = indexNode.Attributes["name"].Value;
				XmlNodeList indexColumnNodeList = indexNode.SelectNodes("Column");
				foreach(XmlNode columnNode in indexColumnNodeList)
				{
					entity.addIndex(indexKey, columnNode.Attributes["name"].Value);
				}
			}

			XmlNodeList commentNodeList = xmlNode.SelectNodes("Comment");
			foreach(XmlNode commentNode in commentNodeList)
			{
				entity.Comment = commentNode.InnerText;
			}

			XmlNodeList extraDDLClauseNodeList = xmlNode.SelectNodes("ExtraDDLClause");
			foreach(XmlNode extraDDLClauseNode in extraDDLClauseNodeList)
			{
				entity.ExtraDDLClause = extraDDLClauseNode.InnerText;
			}
			return entity;
		}
	}
}
