using System;
using System.Xml;

namespace ERModel.Handler
{
	/// <summary>
	/// Summary description for ForeignKeyHandler.
	/// </summary>
	public class ERForeignKeyHandler
	{
		private ERColumnHandler _columnHandler;

		public ERForeignKeyHandler()
		{
			_columnHandler = new ERColumnHandler();
		}

		public ERForeignKey ReadERElement(ERModel model, XmlNode xmlNode)
		{
			ERForeignKey foreignKey = new ERForeignKey(model);
			foreignKey.SetName(xmlNode.Attributes["name"].Value);

			XmlNodeList fkNodeList =  xmlNode.ChildNodes;
			foreach(XmlNode fkNode in fkNodeList)
			{
				if(fkNode.Name.Equals("ReferenceTable"))
				{
					foreignKey.ReferenceTable = fkNode.InnerText;
				}
				else if(fkNode.Name.Equals("ReferencedTable"))
				{
					foreignKey.ReferencedTable = fkNode.InnerText;
				}
				else if(fkNode.Name.Equals("ReferenceSchema"))
				{
					foreignKey.ReferenceSchema = fkNode.InnerText;
				}
				else if(fkNode.Name.Equals("ReferencedSchema"))
				{
					foreignKey.ReferencedSchema = fkNode.InnerText;
				}
				else if(fkNode.Name.Equals("OnUpdate"))
				{
					foreignKey.OnUpdate = fkNode.InnerText;
				}
				else if(fkNode.Name.Equals("OnDelete"))
				{
					foreignKey.OnDelete = fkNode.InnerText;
				}
				else if(fkNode.Name.Equals("Columns"))
				{
					foreignKey.Columns = ReadColumns(fkNode);
				}
				else if(fkNode.Name.Equals("RefColumns"))
				{
					foreignKey.RefColumns = ReadColumns(fkNode);
				}
			}
			return foreignKey;
		}
		
		private string[] ReadColumns(XmlNode xmlNode)
		{
			System.Collections.ArrayList cols = new System.Collections.ArrayList();
			foreach(XmlNode colNode in xmlNode.ChildNodes)
			{
				cols.Add(colNode.InnerText);
			}
			return cols.ToArray(typeof(String)) as string[];
		}
	}
}
