using System;
using System.IO;
using System.Xml;
using System.Reflection;

namespace ERModel.Handler
{
	/// <summary>
	/// Summary description for ERModelHandler.
	/// </summary>
	public class ERModelHandler
	{
		private EREntityHandler _entityHandler;
		private ERForeignKeyHandler _foreignKeyHandler;
		private ERViewHandler _viewHandler;
        private ERProcedureHandler _procedureHandler;
        private ERTriggerHandler _triggerHandler;

		public ERModelHandler()
		{
			_entityHandler = new EREntityHandler();
			_foreignKeyHandler = new ERForeignKeyHandler();
			_viewHandler = new ERViewHandler();
            _procedureHandler = new ERProcedureHandler();
            _triggerHandler = new ERTriggerHandler();
		}

		public ERModel Load(string fileName)
		{
			XmlDocument xmlDoc = new XmlDocument();
			xmlDoc.Load(fileName);
			return ParseDocument(xmlDoc);
		}

		public ERModel Load(Assembly assembly, string name) 
		{
			Stream s = assembly.GetManifestResourceStream(name);
			if (s != null) 
			{
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.Load(s);
				return ParseDocument(xmlDoc);
			} 
			else 
			{
				return null;
			}
		}

		private ERModel ParseDocument(XmlDocument xmlDoc)
		{
			ERModel model = new ERModel();
			if (xmlDoc.SelectSingleNode("Model").Attributes["quotesql"] != null) 
			{
				model.setQuoteSQLIdentifier(int.Parse(xmlDoc.SelectSingleNode("Model").Attributes["quotesql"].Value));
			}
			//for entity
			XmlNodeList entityNodes = xmlDoc.SelectNodes("Model/Entity");
			foreach(XmlNode entityNode in entityNodes)
			{
				model.addElement(_entityHandler.ReadERElement(model, entityNode));
			}

			//for view
			XmlNodeList viewNodes = xmlDoc.SelectNodes("Model/View");
			foreach(XmlNode viewNode in viewNodes)
			{
				model.addElement(_viewHandler.ReadERElement(model, viewNode));
			}
            
            //for procedure
            XmlNodeList procedureNodes = xmlDoc.SelectNodes("Model/Procedure");
            foreach (XmlNode procedureNode in procedureNodes)
            {
                model.addElement(_procedureHandler.ReadERElement(model, procedureNode));
            }

            //for trigger
            XmlNodeList triggerNodes = xmlDoc.SelectNodes("Model/Trigger");
            foreach (XmlNode triggerNode in triggerNodes)
            {
                model.addElement(_triggerHandler.ReadERElement(model, triggerNode));
            }
			
			//for foreign Key
			XmlNodeList foreignKeyNodes = xmlDoc.SelectNodes("Model/ForeignKey");
			foreach(XmlNode fkNode in foreignKeyNodes)
			{
				model.addElement(_foreignKeyHandler.ReadERElement(model, fkNode));
			}

			return model;
		}
	}
}
