using System;
using System.Xml;

namespace ERModel.Handler
{
    public class ERProcedureHandler
    {
        public ERProcedureHandler()
		{
		}

		public ERProcedure ReadERElement(ERModel model, XmlNode xmlNode)
		{
            ERProcedure procedure = new ERProcedure(model);
            procedure.SetName(xmlNode.Attributes["name"].Value);
            string isFunction = xmlNode.Attributes["function"].Value;
            procedure.Function = isFunction!=null && "true".Equals(isFunction.ToLower());
            
            XmlElement contentNode = null;

            foreach (XmlNode childNode in xmlNode.ChildNodes)
            {
                if (childNode is XmlElement)
                {
                    if ("CreateStmt".Equals(childNode.Name))
                    {
                        contentNode = (XmlElement)childNode;
                    }
                    else if ("Parameter".Equals(childNode.Name))
                    {
                        ERProcedureParameter parameter = ReadERProcedureParameter(model, (XmlElement)childNode);
                        if (parameter != null)
                        {
                            procedure.AddParameter(parameter);
                        }
                    }
                }
            }

            if (contentNode != null)
            {
                procedure.Statements = contentNode.InnerText;
            }

            return procedure;
		}

        public ERProcedureParameter ReadERProcedureParameter(ERModel aModel, XmlElement xmlNode)
        {
            ERProcedureParameter parameter = new ERProcedureParameter(aModel);
            parameter.SetName(xmlNode.Attributes["name"].Value);
            parameter.Type = xmlNode.Attributes["type"].Value;
            string direction = xmlNode.Attributes["direction"].Value;
            if (direction == null)
            {
                direction = "in"; // default is "in"
            }
            else
            {
                direction = direction.ToLower();
            }
            if ("out".Equals(direction))
            {
                parameter.Direction = ERProcedureParameter.ParameterDirection.OUT;
            }
            else if ("inout".Equals(direction))
            {
                parameter.Direction = ERProcedureParameter.ParameterDirection.IN_OUT;
            }
            else
            {
                parameter.Direction = ERProcedureParameter.ParameterDirection.IN;
            }
            return parameter;
        }
    }
}
