using System;
using System.Text;
using System.Data;
using NHibernate.Dialect;
using System.Collections;
using ERModel;
using Ormtypemapper;
using NHibernate.SqlTypes;
using NHibernate.Cfg;


namespace ERToDB
{
	/// <summary>
	/// Summary description for DDLGenerator.
	/// </summary>
	public class DDLGenerator
	{
		private TableDDLGenerator _tableDDLGenerator;
		private ForeignKeyDDLGenerator _foreignKeyDDLGenerator;
		private ViewDDLGenerator _viewDDLGenerator;
        private ProcedureDDLGenerator _procedureDDLGenerator;
        private TriggerDDLGenerator _triggerDDLGenerator;

		public DDLGenerator()
		{
			_tableDDLGenerator = new TableDDLGenerator();
			_foreignKeyDDLGenerator = new ForeignKeyDDLGenerator();
			_viewDDLGenerator = new ViewDDLGenerator();
            _procedureDDLGenerator = new ProcedureDDLGenerator();
            _triggerDDLGenerator = new TriggerDDLGenerator();
		}
		
		public string[] createSchema(ERModel.ERModel model, IDictionary properties, string outputPath, bool export)
		{
			Dialect dialect = Dialect.GetDialect(properties);
			ArrayList sqlList = new ArrayList(30);
            ArrayList procedureList = new ArrayList(5);
            ArrayList triggerList = new ArrayList(5);
			foreach(ERElement element in model.Elements)
			{
				if(element is EREntity)
				{
					sqlList.Add(_tableDDLGenerator.generateCreateDDL((EREntity)element, dialect));
					string[] lCommentSQLs = _tableDDLGenerator.generateComment((EREntity) element, dialect);
					for (int i = 0; i < lCommentSQLs.Length; i++) 
					{
						sqlList.Add(lCommentSQLs[i]);
					}
//					Console.WriteLine(_tableDDLGenerator.generateCreateDDL((EREntity)element, dialect));
					System.Collections.IEnumerator indexKeyEnum =  ((EREntity)element).indexKeysEnumerator();
					if(indexKeyEnum != null)
					{
						while(indexKeyEnum.MoveNext())
						{
							string indexName = (string)indexKeyEnum.Current;
							string indexSql = _tableDDLGenerator.GenerateIndex((EREntity)element, dialect, indexName);
//							Console.WriteLine(indexSql);
							sqlList.Add(indexSql);			
						}
					}
				}
				else if(element is ERView)
				{
					sqlList.Add(((ERView)element).CreateStmt);
				}
                else if (element is ERProcedure)
                {
                    procedureList.Add(element);
                }
                else if (element is ERTrigger)
                {
                    triggerList.Add(element);
                }
			}

			foreach(ERElement element in model.Elements)
			{
				if(element is ERForeignKey && dialect.HasAlterTable)
				{
                    string sql = _foreignKeyDDLGenerator.GenerateDDL((ERForeignKey)element, dialect);
                    if (sql != null)
                    {
                        sqlList.Add(sql);
                    }
//					Console.WriteLine(_foreignKeyDDLGenerator.GenerateDDL((ERForeignKey)element, dialect));
				}
			}

			string[] script = GenerateTablesForIDGenerator(model, dialect, null);
			if(script != null)
			for (int i = 0; i < script.Length; i++) 
			{
				sqlList.Add(script[i]);
//				System.Console.WriteLine(script[i]);
			} // for
            
            _procedureDDLGenerator.setDialect(dialect);
            script = _procedureDDLGenerator.GenerateCreateDDL((ERProcedure[])procedureList.ToArray(typeof(ERProcedure)), model.getQuoteSQLIdentifier());
            if (script != null && script.Length>0)
            {
                for (int i = 0; i < script.Length; i++)
                {
                    sqlList.Add(script[i]);
                }
            }

            _triggerDDLGenerator.setDialect(dialect);
            script = _triggerDDLGenerator.GenerateCreateDDL((ERTrigger[])triggerList.ToArray(typeof(ERTrigger)), model.getQuoteSQLIdentifier());
            if (script != null && script.Length > 0)
            {
                for (int i = 0; i < script.Length; i++)
                {
                    sqlList.Add(script[i]);
                }
            }
            
			SQLExport sqlExport = new SQLExport(properties);
			if(outputPath != null)
			{
				sqlExport.DDLOutputFile = outputPath;
				sqlExport.Execute((string[])sqlList.ToArray(typeof(string)), true, export);
			}
			else
			{                
				sqlExport.Execute((string[])sqlList.ToArray(typeof(string)), false, export);
			}

			return sqlList.ToArray(typeof(string)) as string[];
		}

		public string[] DropSchema(ERModel.ERModel model,IDictionary properties, string outputPath, bool export)
		{
			Dialect dialect = Dialect.GetDialect(properties);
			ArrayList sql = new ArrayList();
            ArrayList procedureList = new ArrayList(5);
            ArrayList triggerList = new ArrayList(5);
			try
			{
				if(dialect.DropConstraints)
				{
					foreach(ERElement element in model.Elements)
					{
						if(element is ERForeignKey && dialect.HasAlterTable)
						{
							ERForeignKey foreignKey = element as ERForeignKey;
                            string dropSQL = _foreignKeyDDLGenerator.GenerateDropSQL(foreignKey, dialect, DDLGeneratorUtil.QuoteName(foreignKey.GetName(), dialect, model.getQuoteSQLIdentifier()));
                            if (dropSQL != null)
                            {
                                sql.Add(dropSQL);
                            }
						}
					}
				}

				foreach(ERElement element in model.Elements)
				{
					if(element is EREntity)
					{
						EREntity entity = element as EREntity;
						sql.Add(_tableDDLGenerator.GenerateDropSQL(entity, dialect));
					} 
					else if (element is ERView)
					{
						ERView view = element as ERView;
						sql.Add(_viewDDLGenerator.GenerateDropSQL(view, dialect));
					}
                    else if (element is ERProcedure)
                    {
                        procedureList.Add(element);
                    }
                    else if (element is ERTrigger)
                    {
                        triggerList.Add(element);
                    }
				}

				string[] script = dropTablesForIDGenerator(model, dialect);
				if(script != null)
				for(int i = 0; i < script.Length; i++)
				{
					sql.Add(script[i]);
				}
                _procedureDDLGenerator.setDialect(dialect);
                script = _procedureDDLGenerator.GenerateDropDDL((ERProcedure[])procedureList.ToArray(typeof(ERProcedure)), model.getQuoteSQLIdentifier());
                if (script != null && script.Length > 0)
                {
                    for (int i = 0; i < script.Length; i++)
                    {
                        sql.Add(script[i]);
                    }
                }
                _triggerDDLGenerator.setDialect(dialect);
                script = _triggerDDLGenerator.GenerateDropDDL((ERTrigger[])triggerList.ToArray(typeof(ERTrigger)), model.getQuoteSQLIdentifier());
                if (script != null && script.Length > 0)
                {
                    for (int i = 0; i < script.Length; i++)
                    {
                        sql.Add(script[i]);
                    }
                }
				SQLExport sqlExport = new SQLExport(properties);
				if(outputPath != null)
				{
					sqlExport.DDLOutputFile = outputPath;
					sqlExport.Execute((string[])sql.ToArray(typeof(string)), true, export);
				}
				else
				{
					sqlExport.Execute((string[])sql.ToArray(typeof(string)), false, export);
				}
				return sql.ToArray(typeof(string)) as string[];
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
				return null;
			}
		}

		public string[] DropAndCreateSchema(ERModel.ERModel model,IDictionary properties, string outputPath, bool export)
		{
			Dialect dialect = Dialect.GetDialect(properties);
			ArrayList sqlList = new ArrayList();
			try
			{
				if(dialect.DropConstraints)
				{
					foreach(ERElement element in model.Elements)
					{
						if(element is ERForeignKey)
						{
							ERForeignKey foreignKey = element as ERForeignKey;
							sqlList.Add(_foreignKeyDDLGenerator.GenerateDropSQL(foreignKey, dialect, DDLGeneratorUtil.QuoteName(foreignKey.GetName(), dialect, model.getQuoteSQLIdentifier())));
						}
					}
				}

				foreach(ERElement element in model.Elements)
				{
					if(element is EREntity)
					{
						EREntity entity = element as EREntity;
						sqlList.Add(_tableDDLGenerator.GenerateDropSQL(entity, dialect));
					} 
					else if (element is ERView)
					{
						ERView view = element as ERView;
						sqlList.Add(_viewDDLGenerator.GenerateDropSQL(view, dialect));
					}
				}

				string[] script = dropTablesForIDGenerator(model, dialect);
				if(script != null)
					for(int i = 0; i < script.Length; i++)
					{
						sqlList.Add(script[i]);
					}

				foreach(ERElement element in model.Elements)
				{
					if(element is EREntity)
					{
						sqlList.Add(_tableDDLGenerator.generateCreateDDL((EREntity)element, dialect));
						//						System.Console.WriteLine(_tableDDLGenerator.generateCreateDDL((EREntity)element, dialect));					
					} 
					else if (element is ERView)
					{
						sqlList.Add(((ERView)element).CreateStmt);
					}
				}

				foreach(ERElement element in model.Elements)
				{
					if(element is ERForeignKey)
					{
						sqlList.Add(_foreignKeyDDLGenerator.GenerateDDL((ERForeignKey)element, dialect));
//						System.Console.WriteLine(_foreignKeyDDLGenerator.GenerateDDL((ERForeignKey)element, dialect));
					}
				}

				script = GenerateTablesForIDGenerator(model, dialect, null);
				if(script != null)
					for (int i = 0; i < script.Length; i++) 
					{
						sqlList.Add(script[i]);
//						System.Console.WriteLine(script[i]);
					} // for
				SQLExport sqlExport = new SQLExport(properties);
				if(outputPath != null)
				{
					sqlExport.DDLOutputFile = outputPath;
					sqlExport.Execute((string[])sqlList.ToArray(typeof(string)), true, export);
				}
				else
				{
					sqlExport.Execute((string[])sqlList.ToArray(typeof(string)), false, export);
				}

				return sqlList.ToArray(typeof(string)) as string[];
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine(e.StackTrace);
				return null;
			}
		}

		public string[] UpdateSchema(ERModel.ERModel model, IDbConnection conn, IDictionary properties, string outputPath, bool export)
		{
			ArrayList sql = new ArrayList();
			Dialect dialect = Dialect.GetDialect(properties);
			DataSet.DataSetUtil dataSetUtil = new ERToDB.DataSet.DataSetUtil(conn, dialect);
            ArrayList procedureList = new ArrayList(5);
            ArrayList triggerList = new ArrayList(5);
			foreach(ERElement element in model.Elements)
			{
				if(element is EREntity)
				{
					EREntity entity = element as EREntity;
//					System.Console.WriteLine("entity.GetName()" + entity.GetName());
					DataTable dataTable = dataSetUtil.GetDataTable(conn, entity.Schema, entity.GetName(), dialect, model);
					if(dataTable == null)
					{
						sql.Add(_tableDDLGenerator.generateCreateDDL(entity, dialect));
					}
					else
					{
						string[] generateAlterDDL = _tableDDLGenerator.generateAlterDDL(entity, dataTable, dialect);
						for (int i = 0; i < generateAlterDDL.Length; i++) 
						{
							sql.Add(generateAlterDDL[i]);
						} // for
					}
					string[] lCommentSQLs = _tableDDLGenerator.generateComment((EREntity) element, dialect);
					for (int i = 0; i < lCommentSQLs.Length; i++) 
					{
						sql.Add(lCommentSQLs[i]);
					}
				}
                else if (element is ERProcedure)
                {
                    procedureList.Add(element);
                }
                else if (element is ERTrigger)
                {
                    triggerList.Add(element);
                }
			}//foreach

			//Generate ForeignKey
			foreach(ERElement element in model.Elements)
			{
				if(element is ERForeignKey)
				{
					ERForeignKey foreignKey = element as ERForeignKey;
					DataTable dataTable = dataSetUtil.GetDataTable(conn, foreignKey.ReferencedSchema, foreignKey.ReferencedTable, dialect, model);
					if(dialect.HasAlterTable)
					{
						if(dataTable == null || dataTable.Constraints[foreignKey.GetName()] == null)
						{
							sql.Add(_foreignKeyDDLGenerator.GenerateDDL(foreignKey, dialect));
						}
					}
				}
			}//foreach

			string[] script = GenerateTablesForIDGenerator(model, dialect, dataSetUtil);
			if(script != null)
			for(int i = 0; i < script.Length; i++)
			{
				sql.Add(script[i]);
			}
            _procedureDDLGenerator.setDialect(dialect);
            script = _procedureDDLGenerator.GenerateUpdateDDL((ERProcedure[])procedureList.ToArray(typeof(ERProcedure)), model.getQuoteSQLIdentifier());
            if (script != null && script.Length > 0)
            {
                for (int i = 0; i < script.Length; i++)
                {
                    sql.Add(script[i]);
                }
            }
            _triggerDDLGenerator.setDialect(dialect);
            script = _triggerDDLGenerator.GenerateUpdateDDL((ERTrigger[])triggerList.ToArray(typeof(ERTrigger)), model.getQuoteSQLIdentifier());
            if (script != null && script.Length > 0)
            {
                for (int i = 0; i < script.Length; i++)
                {
                    sql.Add(script[i]);
                }
            }
			SQLExport sqlExport = new SQLExport(properties);
			if(outputPath != null)
			{
				sqlExport.DDLOutputFile = outputPath;
				sqlExport.Execute((string[])sql.ToArray(typeof(string)), true, export);
			}
			else
			{
				sqlExport.Execute((string[])sql.ToArray(typeof(string)), false, export);
			}

			return sql.ToArray(typeof(string)) as string[];
		}

		private string[] GenerateTablesForIDGenerator(ERModel.ERModel aModel, Dialect aDialect, DataSet.DataSetUtil dataSetUtil){
			ArrayList lSQL = new ArrayList();
			ArrayList lGenerated = new ArrayList();
			foreach(ERElement element in aModel.Elements){
            ERElement lElement = element;
            if (lElement is EREntity) {
                ArrayList pks = new ArrayList();
				IEnumerator elementEnum = ((EREntity)lElement).GetElementsEnumerator();
				while(elementEnum.MoveNext()){
                    ERElement lElement2 = (ERElement) elementEnum.Current;
                    if (lElement2 is ERColumn) {
                        ERColumn lColumn = (ERColumn) lElement2;
                        // Only primary key and not foreign key can use id generator 
                        if (lColumn.PrimaryKey && !lColumn.ForeignKey) {
                            pks.Add(lColumn);
                        }
                    }
                }//for
                
                // Id generator works with single primary key only
                if (pks.Count == 1) {
                    ERColumn lColumn = (ERColumn) pks[0];
					String lIdGenerator = DDLGeneratorUtil.GetIDGenerator(lColumn.IdGenerator, aDialect, DBTypeUtil.isIntType(lColumn.Type));
					if (NeedTableForGenerator(lIdGenerator, aDialect)) {
						bool lTableQuote = DDLGeneratorUtil.isQuote(lColumn.GetEntity().GetName(), aDialect, aModel.getQuoteSQLIdentifier());
						bool lColumnQuote = DDLGeneratorUtil.isQuote(lColumn.GetName(), aDialect, aModel.getQuoteSQLIdentifier());
                        string lIdGeneratorKey = Orm.Util.DDLGeneratorUtil.GetIDGeneratorKey(lColumn.GetEntity().Schema, lColumn.GetEntity().GetName(), lColumn.GetName(), lIdGenerator,
							lColumn.IdGeneratorKey, aDialect,lTableQuote ,lColumnQuote);
                        if (!lGenerated.Contains(lIdGeneratorKey)) {
                            if (dataSetUtil == null ||
                                (!dataSetUtil.IsSequence(lIdGeneratorKey) && !dataSetUtil.IsTable(lIdGeneratorKey, aModel))) {
                                String[] lScript = GenerateTableForIDGenerator(lIdGenerator, lIdGeneratorKey,
                                    DDLGeneratorUtil.GetDefaultIDTableColumnName(), aDialect);
                                if (lScript != null) {
                                    for (int i = 0; i < lScript.Length; i++) {
                                        lSQL.Add(lScript[i]);
                                    } // for
                                }
                                lGenerated.Add(lIdGeneratorKey);
                            }
                        }
                    }
                }
            }
        }
        
        return lSQL.ToArray(typeof(string)) as string[];
    }

		private bool NeedTableForGenerator(String aStrategy, Dialect aDialect) {
			if (aStrategy.Equals(ERModelConstants.IDG_HILO) || aStrategy.Equals(ERModelConstants.IDG_SEQUENCE)
				|| aStrategy.Equals(ERModelConstants.IDG_SEQ_HILO)) {
				return true;
			}
			return false;
		}

		private string[] GenerateTableForIDGenerator(string aStrategy, string aGeneratorKey, string aColumn, Dialect aDialect){
			if (aStrategy.Equals("sequence")) {
				return aDialect.GetCreateSequenceStrings(aGeneratorKey);
			} else if (aStrategy.Equals("hilo") || aStrategy.Equals("seqhilo")) {
				return new string[] {"create table " + aGeneratorKey + " ( " + aColumn + " " + aDialect.GetTypeName(SqlTypeFactory.Int32)
					+ " )\n" + "insert into " + aGeneratorKey + " values ( 0 )"};
			}
			return null;
		}

		private String[] dropTablesForIDGenerator(ERModel.ERModel aModel, Dialect aDialect) 
		{
			ArrayList lSQL = new ArrayList();
			IList lGenerated = new ArrayList();
			
			foreach(ERElement lElement in aModel.Elements)
			{
				if (lElement is EREntity) 
				{
					ArrayList pks = new ArrayList();
					EREntity entity = lElement as EREntity;
					foreach(ERElement lElement2 in entity.Elements)
					{
						if (lElement2 is ERColumn) 
						{
							ERColumn lColumn = lElement2 as ERColumn;
							// Only primary key and not foreign key can use id generator 
							if (lColumn.PrimaryKey && !lColumn.ForeignKey) 
							{
								pks.Add(lColumn);
							}
						}
					}
                
					// Id generator works with single primary key only
					if (pks.Count == 1) 
					{
						ERColumn lColumn = (ERColumn) pks[0];
						string lIdGenerator = DDLGeneratorUtil.GetIDGenerator(lColumn.IdGenerator, aDialect, DBTypeUtil.isIntType(lColumn.Type));
						if (NeedTableForGenerator(lIdGenerator, aDialect)) 
						{
							bool lTableQuote = DDLGeneratorUtil.isQuote(lColumn.GetEntity().GetName(), aDialect, aModel.getQuoteSQLIdentifier());
							bool lColumnQuote = DDLGeneratorUtil.isQuote(lColumn.GetName(), aDialect, aModel.getQuoteSQLIdentifier());
							string lIdGeneratorKey = Orm.Util.DDLGeneratorUtil.GetIDGeneratorKey(lColumn.GetEntity().Schema, lColumn.GetEntity().GetName(), lColumn.GetName(), lIdGenerator, lColumn.IdGeneratorKey, aDialect, lTableQuote, lColumnQuote);
							if (!lGenerated.Contains(lIdGeneratorKey)) 
							{
								string[] lScript = dropTableForIDGenerator(lIdGenerator, lIdGeneratorKey,
									DDLGeneratorUtil.GetDefaultIDTableColumnName(), aDialect);
								if (lScript != null) 
								{
									for (int i = 0; i < lScript.Length; i++) 
									{
										lSQL.Add(lScript[i]);
									} // for
								}
								lGenerated.Add(lIdGeneratorKey);
							}
						}
					}
				}
			}
        
			return lSQL.ToArray(typeof(string)) as string[];
		}
		
		private string[] dropTableForIDGenerator(string aStrategy, string aGeneratorKey, string aColumn, Dialect aDialect) 
		{
			if (aStrategy.Equals("sequence")) 
			{
				return new string[]{aDialect.GetDropSequenceString(aGeneratorKey)};
			} 
			else if (aStrategy.Equals("hilo") || aStrategy.Equals("seqhilo")) 
			{
				return new String[] { aDialect.GetDropTableString(aGeneratorKey) };
			}
			return null;
		}
	}
}
