using System;
using NHibernate.Dialect;
using ERModel;

namespace ERToDB
{
	/// <summary>
	/// Summary description for DDLGeneratorUtil.
	/// </summary>
	public class DDLGeneratorUtil
	{
		public DDLGeneratorUtil()
		{
		}

		    
		public static string GetQualifiedName(String schema, String table, Dialect aDialect, int aQuoteSQL) 
		{
			String lSchema = QuoteName(schema, aDialect, aQuoteSQL);
			String lTable = QuoteName(table, aDialect, aQuoteSQL);
			if (lSchema == null || lSchema.Length == 0) 
			{
				return lTable;
			} 
			else 
			{
				return lSchema + "." + lTable;
			}
		}

		public static string QuoteName(string s, Dialect dialect, int quoteSQL) 
		{
			if (s == null || s.Length == 0) 
			{
				return s;
			}
			if (quoteSQL == ERModel.ERModel.QUOTE_SQL_YES
				|| (quoteSQL == ERModel.ERModel.QUOTE_SQL_AUTO 
					&& (SQLReservedWordChecker.getInstance().isReservedWord(s)
                    || !IsValidSQLName(s))))
			{
				return dialect.OpenQuote + s + dialect.CloseQuote;
			} 
			else 
			{
				return s;
			}
		}

		public static bool isQuote(string s, Dialect dialect, int quoteSQL)
		{
			if (quoteSQL == ERModel.ERModel.QUOTE_SQL_YES
				|| (quoteSQL == ERModel.ERModel.QUOTE_SQL_AUTO 
				&& (SQLReservedWordChecker.getInstance().isReservedWord(s)
				|| !IsValidSQLName(s))))
			{
				return true;
			} 
			return false;
		}

        public static bool IsValidSQLName(string s)
        {
            if (s == null || s.Length == 0)
            {
                return false;
            }
            foreach (char ch in s)
            {
                bool valid = Char.IsLetterOrDigit(ch) || ch == '_';
                if (!valid)
                {
                    return false;
                }
            }
            return true;
        }

		public static string GetIDGeneratorKey(string schema, string tableName, string idGenerator, string idGeneratorKey) 
		{
			if (idGeneratorKey != null && idGeneratorKey.Length > 0)
				return idGeneratorKey;
        
			string prefix = "";
			if (schema != null && schema.Length > 0) 
			{
				prefix = schema + ".";
			}
			if (idGenerator.Equals(ERModelConstants.IDG_SEQUENCE) || idGenerator.Equals(ERModelConstants.IDG_SEQ_HILO)) 
			{
				return prefix + "seq_" + tableName.Replace(" ", "_");
			} 
			else if (idGenerator.Equals(ERModelConstants.IDG_HILO)) 
			{
				return prefix + "id_" + tableName.Replace(" ", "_");
			}
        
			return null;
		}

		public static string GetIDGenerator(String aStrategy, Dialect aDialect, bool isInt) 
		{
			if (aStrategy.Equals(ERModelConstants.IDG_NATIVE)) 
			{
				if (isInt) 
				{
					if (aDialect.SupportsIdentityColumns) 
					{
						return ERModelConstants.IDG_IDENTITY;
					} 
					else if (aDialect.SupportsSequences) 
					{
						return ERModelConstants.IDG_SEQUENCE;
					} 
					else 
					{
						return ERModelConstants.IDG_HILO;
					}
				} 
				else 
				{
					return ERModelConstants.IDG_ASSIGNED;
				}
			}

			return aStrategy;
		}

		public static string GetDefaultIDTableColumnName() 
		{
			return "nextval";
		}
	}
}
