using System;
using System.Collections;
using System.Text;
using ERModel;

namespace ERToDB.Decorators
{
    public class MySQLDecorator : StandardDecorator
    {

        public override string[] DecorateCreateSQL(ERModel.ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList(aProcedures.Length*2);
            for (int i = 0; i < aProcedures.Length; i++)
            {
                ERProcedure procedure = aProcedures[i];
                StringBuilder sb = new StringBuilder();
                sb.Append("DROP ");
                sb.Append(procedure.Function ? "FUNCTION " : "PROCEDURE ");
                sb.Append("IF EXISTS ");
                sb.Append(DDLGeneratorUtil.QuoteName(procedure.GetName(), aDialect, aQuoteSQL));
                result.Add(sb.ToString());
                result.Add(procedure.Statements);
            }
            return (string[]) result.ToArray(typeof(string));
        }

        public override string[] DecorateCreateSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList(aTriggers.Length * 2);
            for (int i = 0; i < aTriggers.Length; i++)
            {
                ERTrigger trigger = aTriggers[i];
                StringBuilder sb = new StringBuilder();
                sb.Append("DROP TRIGGER IF EXISTS ");
                sb.Append(DDLGeneratorUtil.QuoteName(trigger.GetName(), aDialect, aQuoteSQL));
                result.Add(sb.ToString());
                result.Add(trigger.Statements);
            }
            return (string[])result.ToArray(typeof(string));
        }

        public override string[] DecorateDropSQL(ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList(aProcedures.Length);
            for (int i = 0; i < aProcedures.Length; i++)
            {
                ERProcedure procedure = aProcedures[i];
                StringBuilder sb = new StringBuilder();
                sb.Append("DROP ");
                sb.Append(procedure.Function ? "FUNCTION " : "PROCEDURE ");
                sb.Append("IF EXISTS ");
                sb.Append(DDLGeneratorUtil.QuoteName(procedure.GetName(), aDialect, aQuoteSQL));
                result.Add(sb.ToString());
            }
            return (string[])result.ToArray(typeof(string));
        }

        public override string[] DecorateDropSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList(aTriggers.Length);
            for (int i = 0; i < aTriggers.Length; i++)
            {
                ERTrigger trigger = aTriggers[i];
                StringBuilder sb = new StringBuilder();
                sb.Append("DROP TRIGGER IF EXISTS ");
                sb.Append(DDLGeneratorUtil.QuoteName(trigger.GetName(), aDialect, aQuoteSQL));
                result.Add(sb.ToString());
            }
            return (string[])result.ToArray(typeof(string));
        }
    }
}
