using System;
using System.Collections;
using System.Text;
using ERModel;
using System.Text.RegularExpressions;

namespace ERToDB.Decorators
{
    public class PostgreSQLDecorator : StandardDecorator
    {
        public override string[] DecorateCreateSQL(ERModel.ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList(aProcedures.Length*2);
            for (int i=0;i<aProcedures.Length;i++)
            {
                ERProcedure procedure = aProcedures[i];                    
                StringBuilder drop = new StringBuilder("DROP FUNCTION IF EXISTS ");
                drop.Append(DDLGeneratorUtil.QuoteName(procedure.GetName(), aDialect, aQuoteSQL));
                drop.Append("(");
                IEnumerator parameters = procedure.GetElementsEnumerator();
                if (parameters != null)
                {
                    bool first = true;
                    while (parameters.MoveNext())
                    {
                        if (first)
                        {
                            first = false;
                        }
                        else
                        {
                            drop.Append(",");
                        }
                        ERProcedureParameter parameter = (ERProcedureParameter) parameters.Current;
                        drop.Append(parameter.Type);
                    }
                }
                drop.Append(")");
                result.Add(drop.ToString());
                result.Add(procedure.Statements);
            }
            return (string[])result.ToArray(typeof(string));
        }

        public override string[] DecorateDropSQL(ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList();
            for (int i=0;i<aProcedures.Length;i++)
            {
                ERProcedure procedure = aProcedures[i];
                StringBuilder drop = new StringBuilder("DROP FUNCTION IF EXISTS ");
                drop.Append(DDLGeneratorUtil.QuoteName(procedure.GetName(), aDialect, aQuoteSQL));
                drop.Append("(");
                IEnumerator parameters = procedure.GetElementsEnumerator();
                if (parameters != null)
                {
                    bool first = true;
                    while (parameters.MoveNext())
                    {
                        if (first)
                        {
                            first = false;
                        }
                        else
                        {
                            drop.Append(",");
                        }
                        ERProcedureParameter parameter = (ERProcedureParameter)parameters.Current;
                        drop.Append(parameter.Type);
                    }
                }
                drop.Append(")");
                result.Add(drop.ToString());
            }
            return (string[])result.ToArray(typeof(string));
        }

        public override string[] DecorateCreateSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList(aTriggers.Length*2);
            for (int i = 0; i < aTriggers.Length; i++)
            {
                ERTrigger trigger = aTriggers[i];
                StringBuilder sb = new StringBuilder("DROP TRIGGER IF EXISTS ");
                sb.Append(DDLGeneratorUtil.QuoteName(trigger.GetName(), aDialect, aQuoteSQL));
                sb.Append(" ON ");
                sb.Append(DDLGeneratorUtil.QuoteName(ExtraTableNameFromCreateStatement(trigger.Statements), aDialect, aQuoteSQL));
                result.Add(sb.ToString());
                result.Add(trigger.Statements);
            }
            return (string[])result.ToArray(typeof(string));
        }

        public override string[] DecorateDropSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList();
            for (int i = 0; i < aTriggers.Length; i++)
            {
                ERTrigger trigger = aTriggers[i];
                result.Add("DROP TRIGGER IF EXISTS " + DDLGeneratorUtil.QuoteName(trigger.GetName(), aDialect, aQuoteSQL)+" ON "
                    +DDLGeneratorUtil.QuoteName(trigger.Statements, aDialect, aQuoteSQL));
            }
            return (string[])result.ToArray(typeof(string));
        }

        public static string ExtraTableNameFromCreateStatement(string aCreateStatement)
        {
            Regex ex = new Regex("\\son\\s(.+)?\\s", RegexOptions.Multiline | RegexOptions.IgnoreCase);
            Match match = ex.Match(aCreateStatement);
            if (match.Success)
            {
                return match.Groups[1].Value;
            }
            return null;
        }
    }
}
