using System;
using System.Collections;
using System.Text;
using ERModel;

namespace ERToDB.Decorators
{
    public class StandardDecorator : Decorator
    {
        #region Decorator Members

        public virtual string[] DecorateCreateSQL(ERModel.ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList();
            for (int i=0;i<aProcedures.Length;i++) 
            {
                ERProcedure procedure = aProcedures[i];
                result.Add(procedure.Statements);
            }
            return (string[]) result.ToArray(typeof(string));
        }

        public virtual string[] DecorateDropSQL(ERModel.ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList();
            for (int i = 0; i < aProcedures.Length; i++)
            {
                ERProcedure procedure = aProcedures[i];
                result.Add("drop "+(procedure.Function?"function":"procedure")+" "+DDLGeneratorUtil.QuoteName(procedure.GetName(), aDialect, aQuoteSQL));
            }
            return (string[])result.ToArray(typeof(string));
        }

        public virtual string[] DecorateAlterSQL(ERModel.ERProcedure[] aProcedures, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            return DecorateCreateSQL(aProcedures, aDialect, aQuoteSQL);
        }

        public virtual string[] DecorateCreateSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList();
            for (int i = 0; i < aTriggers.Length; i++)
            {
                ERTrigger trigger = aTriggers[i];
                result.Add(trigger.Statements);
            }
            return (string[])result.ToArray(typeof(string));
        }

        public virtual string[] DecorateDropSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            ArrayList result = new ArrayList();
            for (int i = 0; i < aTriggers.Length; i++)
            {
                ERTrigger trigger = aTriggers[i];
                StringBuilder sb = new StringBuilder("drop trigger ");
                if (aDialect.SupportsIfExistsBeforeTableName)
                {
                    sb.Append("if exists ");
                }
                sb.Append(DDLGeneratorUtil.QuoteName(trigger.GetName(), aDialect, aQuoteSQL));
                if (aDialect.SupportsIfExistsAfterTableName)
                {
                    sb.Append(" if exists");
                }
                result.Add(sb.ToString());
            }
            return (string[])result.ToArray(typeof(string));
        }

        public virtual string[] DecorateAlterSQL(ERTrigger[] aTriggers, NHibernate.Dialect.Dialect aDialect, int aQuoteSQL)
        {
            return DecorateCreateSQL(aTriggers, aDialect, aQuoteSQL);
        }

        #endregion
    }
}
