using System;
using System.Text;
using ERModel;
using NHibernate.Dialect;

namespace ERToDB
{
	/// <summary>
	/// Summary description for ForeignKeyDDLGenerator.
	/// </summary>
	public class ForeignKeyDDLGenerator
	{
		public ForeignKeyDDLGenerator()
		{
		}

		public string GenerateDDL(ERForeignKey foreignKey, Dialect dialect)
		{
			StringBuilder sb = new StringBuilder();
			int quoteSqlIdentifier = foreignKey.GetModel().getQuoteSQLIdentifier();
			string fkName = foreignKey.GetName();
			sb.Append("alter table ");			
			sb.Append(DDLGeneratorUtil.GetQualifiedName(foreignKey.ReferenceSchema, foreignKey.ReferenceTable, dialect, quoteSqlIdentifier));
			
			string[] pks = new string[foreignKey.RefColumns.Length];
			string[] fks = new string[foreignKey.Columns.Length];

			for(int i = 0; i < pks.Length; i++)
			{
				pks[i] = DDLGeneratorUtil.QuoteName(foreignKey.RefColumns[i], dialect, quoteSqlIdentifier);
			}

			for(int i = 0; i < fks.Length; i++)
			{
				fks[i] = DDLGeneratorUtil.QuoteName(foreignKey.Columns[i], dialect, quoteSqlIdentifier);
			}

            string addForeignKeyConstraintString = dialect.GetAddForeignKeyConstraintString(DDLGeneratorUtil.QuoteName(fkName, dialect, quoteSqlIdentifier), fks,
                DDLGeneratorUtil.GetQualifiedName(foreignKey.ReferencedSchema, foreignKey.ReferencedTable, dialect, quoteSqlIdentifier), pks);

            if (addForeignKeyConstraintString == null)
            {
                return null;
            }

            sb.Append(addForeignKeyConstraintString);
			sb.Append(dialect.GetReferenceOnUpdateDeleteString(foreignKey.OnUpdate, foreignKey.OnDelete));

			return sb.ToString();
		}

		public string GenerateDropSQL(ERForeignKey foreignKey, Dialect dialect, string name) 
		{
            string dropForeignKeyConstraintString = dialect.GetDropForeignKeyConstraintString(name);
            if (dropForeignKeyConstraintString == null)
            {
                return null;
            }
			string dropSql = "alter table " + DDLGeneratorUtil.GetQualifiedName(foreignKey.ReferenceSchema, foreignKey.ReferenceTable, dialect, foreignKey.GetModel().getQuoteSQLIdentifier())
                + dropForeignKeyConstraintString;
			return dropSql;
		}
	}
}
