using System;
using System.Text;
using NHibernate.Dialect;
using ERModel;
using ERToDB.Decorators;

namespace ERToDB
{
    public class ProcedureDDLGenerator
    {

        private Dialect _dialect;

        public Dialect GetDialect()
        {
            return _dialect;
        }

        public void setDialect(Dialect aDialect)
        {
            _dialect = aDialect;
            _decorator = GetSQLDecorator(aDialect);
        }

        private Decorator _decorator;

        public Decorator GetDecorator()
        {
            return _decorator;
        }

        public ProcedureDDLGenerator()
        {
        }

        public string[] GenerateCreateDDL(ERProcedure[] aProcedures, int aQuoteSQL)
        {
            return _decorator.DecorateCreateSQL(aProcedures, _dialect, aQuoteSQL);
        }

        public string[] GenerateDropDDL(ERProcedure[] aProcedures, int aQuoteSQL)
        {
            return _decorator.DecorateDropSQL(aProcedures, _dialect, aQuoteSQL);
        }

        public string[] GenerateUpdateDDL(ERProcedure[] aProcedures, int aQuoteSQL)
        {
            return _decorator.DecorateAlterSQL(aProcedures, _dialect, aQuoteSQL);
        }

        public static Decorator GetSQLDecorator(Dialect aDialect)
        {
            if (aDialect is MySQLDialect)
            {
                return new MySQLDecorator();
            }
            if (aDialect is Oracle9Dialect)
            {
                return new OracleDecorator();
            }
            if (aDialect is SybaseDialect)
            {
                return new SybaseDecorator();
            }
            if (aDialect is PostgreSQLDialect)
            {
                return new PostgreSQLDecorator();
            }
            return new StandardDecorator();
        }


    }
}
