using System;
using System.Collections;
using System.Data;
using System.Text;
using System.IO;
using NHibernate.Cfg;
using NHibernate.Connection;
using NHibernate.Dialect;
using NHibernate.Util;
using ERToDB.Configuration;


namespace ERToDB
{
	/// <summary>
	/// Summary description for SQLExport.
	/// </summary>
	public class SQLExport
	{
		private IDictionary _connProperties;
		private Dialect _dialect;
		private string _delimiter;
		private string _ddlOutputFile;

		public SQLExport(IDictionary properties)
		{
			_dialect = Dialect.GetDialect(properties);
			_connProperties = properties;
		}

		public string Delimiter
		{
			get {return _delimiter;}
			set {_delimiter = value;}
		}

		public string DDLOutputFile
		{
			get {return _ddlOutputFile;}
			set {_ddlOutputFile = value;}
		}

		public void Execute(string[] sqls, bool writeFile, bool export)
		{
			IDbConnection connection = null;
			IConnectionProvider connectionProvider = null;
			IDbCommand statement = null;

			IDictionary props = new Hashtable();
			foreach(DictionaryEntry de in _dialect.DefaultProperties)
			{
				props[ de.Key ] = de.Value;
			}

			if(_connProperties != null)
			{
				foreach( DictionaryEntry de in _connProperties )
				{
					props[ de.Key ] = de.Value;
				}
			}

			if(export)
			{
				try
				{
					connectionProvider = ConnectionProviderFactory.NewConnectionProvider(props);
					connection = connectionProvider.GetConnection();
					statement = connection.CreateCommand();
				}
				catch(Exception e)
				{
					Console.WriteLine(e.Message + ":" + e.StackTrace);
				}
			}
			
			StreamWriter fileOutput = null;
			if(writeFile)
			{
				if(_ddlOutputFile != null)
				{
					fileOutput = new StreamWriter( _ddlOutputFile );
				}
			}

			for(int i = 0; i < sqls.Length; i++)
			{
				try
				{
					string formatted;
					if(writeFile)
					{
						formatted = Format(sqls[i]);
					}
					else
					{
						formatted = sqls[i];
					}

					if(_delimiter != null)
					{
						formatted += _delimiter;
					}
					if(writeFile)
					{
						fileOutput.WriteLine(formatted);
					}
					
					if(export)
					{
                        if (!"go".Equals(sqls[i].ToLower()))
                        {
                            statement.CommandText = sqls[i];
                            statement.CommandType = CommandType.Text;
                            statement.ExecuteNonQuery();
                        }						
					}
				}
				catch(Exception e)
				{
					Console.WriteLine(e.Message +  " : " + e.StackTrace);
				}
			}
			// end for
			try
			{
				if( statement != null )
				{
					statement.Dispose();
				}
				if( connection != null )
				{
					connectionProvider.CloseConnection( connection );
					connectionProvider.Dispose();
				}
				if(fileOutput != null)
				{
					fileOutput.Close();
				}
			}
			catch(Exception e)
			{
				Console.Error.WriteLine( "Could not close connection: " + e.Message );
			}
		}

		private static string Format( string sql )
		{
			if( sql.IndexOf( "\"" ) > 0 || sql.IndexOf( "'" ) > 0 )
			{
				return sql;
			}

			string formatted;

			if( sql.ToLower().StartsWith( "create table" ) )
			{
				StringBuilder result = new StringBuilder( 60 );
				StringTokenizer tokens = new StringTokenizer( sql, "(,)", true );

				int depth = 0;

				foreach( string tok in tokens )
				{
					if( StringHelper.ClosedParen.Equals( tok ) )
					{
						depth--;
						if( depth == 0 )
						{
							result.Append( "\n" );
						}
					}
					result.Append( tok );
					if( StringHelper.Comma.Equals( tok ) && depth == 1 )
					{
						result.Append( "\n  " );
					}
					if( StringHelper.OpenParen.Equals( tok ) )
					{
						depth++;
						if( depth == 1 )
						{
							result.Append( "\n  " );
						}
					}
				}

				formatted = result.ToString();
			}
			else
			{
				formatted = sql;
			}

			return formatted;
		}
	}
}
