using System;
using System.Data;
using NHibernate;
using NHibernate.Impl;
using NHibernate.Cfg;
using NHibernate.Dialect;

namespace Orm
{
	public class PersistentSession
	{
		private readonly ISession _session;
		private bool _transaction = false;
		
		public PersistentSession(ISession aSession, Configuration cfg)
		{
			_session = aSession;
			Initialize(cfg);
		}
		
		protected virtual void Initialize(Configuration cfg) {
			String dialectClass = cfg.GetProperty("hibernate.dialect");
			if (dialectClass!=null) {
				if ("NHibernate.Dialect.MsSql7Dialect".Equals(dialectClass)
				    || "NHibernate.Dialect.MsSql2000Dialect".Equals(dialectClass)
				   ) {
					String nocount = cfg.GetProperty("mssql.nocount");
					if (nocount!=null && "false".Equals(nocount)) {
						IDbCommand command = _session.Connection.CreateCommand();
						command.CommandText = "set nocount off";
						command.ExecuteNonQuery();				
					}
				}
			}
		}

		public PersistentTransaction BeginTransaction()
		{
			try
			{
				PersistentTransaction lPersistentTransaction = new PersistentTransaction(_session.BeginTransaction(), this);
				IsTransaction = true;
				return lPersistentTransaction;
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

//		public void CancelQuery()
//		{
//			try
//			{
//				_session.can
//			}
//			catch(HibernateException e)
//			{
//				throw new PersistentException(e);
//			}
//		}
		
		public void Close()
		{
			try
			{
				_session.Close();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public System.Data.IDbConnection Connection()
		{
			try
			{
				return _session.Connection;
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public bool Contains(object aObject)
		{
			return _session.Contains(aObject);
		}

		public ICriteria CreateCriteria(Type persistentClass)
		{
			return _session.CreateCriteria(persistentClass);
		}
		
		public ICriteria CreateCriteria(System.Type persistentClass, string alias)
		{
			return _session.CreateCriteria(persistentClass, alias);
		}
		
		public IQuery CreateFilter(object collection,string queryString)
		{
			try
			{
				return _session.CreateFilter(collection, queryString);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

        public IQuery CreateFilter(Orm.Util.ORMCollection collection, string queryString)
        {
            return CreateFilter(collection.GetCollection(), queryString);
        }

        public IQuery CreateFilter(Orm.Util.ORMSet collection, string queryString)
        {
            return CreateFilter(collection.GetSet(), queryString);
        }

        public IQuery CreateFilter(Orm.Util.ORMMap collection, string queryString)
        {
            return CreateFilter(collection.GetMap(), queryString);
        }

        public IQuery CreateFilter(Orm.Util.ORMList collection, string queryString)
        {
            return CreateFilter(collection.GetList(), queryString);
        }

		public IQuery CreateQuery(string queryString)
		{
			try
			{
				return _session.CreateQuery(queryString);
			}
			catch(Exception e)
			{
				throw new PersistentException(e);
			}
		}

		public IQuery CreateSQLQuery(string sql, string returnAlias, Type returnClass)
		{
			try
			{
				return _session.CreateSQLQuery(sql, returnAlias, returnClass);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public IQuery CreateSQLQuery(string sql, string[] returnAliases, Type[] returnClasses)
		{
			try
			{
				return _session.CreateSQLQuery(sql, returnAliases, returnClasses);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public ISQLQuery CreateSQLQuery(string queryString) 
		{
			try
			{
				return _session.CreateSQLQuery(queryString);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Delete(object obj)
		{
			try
			{
				_session.Delete(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public int Delete(string query)
		{
			try
			{
				return _session.Delete(query);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		//The Delete method have four type of parameters input

		public override int GetHashCode()
		{
			return _session.GetHashCode();
		}

		public override bool Equals(object obj)
		{
			return _session.Equals(obj);
		}
		
		public void Evict(object obj)
		{
			try
			{
				_session.Evict(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

        public void Clear()
        {
            try
            {
                _session.Clear();
            }
            catch (HibernateException e)
            {
                throw new PersistentException(e);
            }
        }

		public void Flush()
		{
			try
			{
				_session.Flush();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public object Get(Type clazz,object id)
		{
			try
			{
				return _session.Get(clazz, id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public object Get(Type clazz, object id, LockMode lockMode)
		{
			try
			{
				return _session.Get(clazz, id, lockMode);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public LockMode GetCurrentLockMode(object obj)
		{
			try
			{
				return _session.GetCurrentLockMode(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public object GetIdentifier(object obj)
		{
			try
			{
				return _session.GetIdentifier(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public IQuery GetNamedQuery(string queryName)
		{
			try
			{
				return _session.GetNamedQuery(queryName);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public bool IsConnected
		{
			get {return _session.IsConnected;}
		}

		public bool IsOpen
		{
			get {return _session.IsOpen;}
		}
		
		public void Load(object obj, object id)
		{
			try
			{
				_session.Load(obj, id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public object Load(Type theType, object id)
		{
			try
			{
				return _session.Load(theType, id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Load(Type theType, object id, LockMode lockMode)
		{
			try
			{
				_session.Load(theType, id, lockMode);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

#if NET_2_0
		/// <summary>
		/// Return the persistent instance of the given entity class with the given identifier,
		/// obtaining the specified lock mode.
		/// </summary>
		/// <typeparam name="T">A persistent class</typeparam>
		/// <param name="id">A valid identifier of an existing persistent instance of the class</param>
		/// <param name="lockMode">The lock level</param>
		/// <returns>the persistent instance</returns>
		public T Load<T>(object id, LockMode lockMode)
		{
			try
			{
				return _session.Load<T>(id, lockMode);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		/// <summary>
		/// Return the persistent instance of the given entity class with the given identifier,
		/// assuming that the instance exists.
		/// </summary>
		/// <remarks>
		/// You should not use this method to determine if an instance exists (use a query or
		/// <see cref="Get{T}(object)" /> instead). Use this only to retrieve an instance that you
		/// assume exists, where non-existence would be an actual error.
		/// </remarks>
		/// <typeparam name="T">A persistent class</typeparam>
		/// <param name="id">A valid identifier of an existing persistent instance of the class</param>
		/// <returns>The persistent instance or proxy</returns>
		public T Load<T>(object id)
		{
			try
			{
				return _session.Load<T>(id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
#endif
		public void Lock(object obj, LockMode lockMode)
		{
			try
			{
				_session.Lock(obj, lockMode);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public void Reconnect()
		{
			try
			{
				_session.Reconnect();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Reconnect(System.Data.IDbConnection connection)
		{
			try
			{
				_session.Reconnect(connection);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public void Refresh(object obj)
		{
			try
			{
				_session.Refresh(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Replicate(object obj, ReplicationMode replicationMode)
		{
			try
			{
				_session.Replicate(obj, replicationMode);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public object Save(object obj)
		{
			try
			{
				return _session.Save(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Save(object obj, object id)
		{
			try
			{
				_session.Save(obj, id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public void SaveOrUpdate(object obj)
		{
			try
			{
				_session.SaveOrUpdate(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public object SaveOrUpdateCopy(object obj)
		{
			try
			{
				return _session.SaveOrUpdateCopy(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public object SaveOrUpdateCopy(object obj, object id)
		{
			try
			{
				return _session.SaveOrUpdateCopy(obj, id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public void Update(object obj)
		{
			try
			{
				_session.Update(obj);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public bool IsTransaction
		{
			get { return _transaction; }
			set { _transaction= value; }
		}

#if NET_2_0
		/// <summary>
		/// Strongly-typed version of <see cref="Get(System.Type, object)" />
		/// </summary>
		public T Get<T>(object id)
		{
			try
			{
				return _session.Get<T>(id);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		/// <summary>
		/// Strongly-typed version of <see cref="Get(System.Type, object, LockMode)" />
		/// </summary>
		public T Get<T>(object id, LockMode lockMode)
		{
			try
			{
				return _session.Get<T>(id, lockMode);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
#endif

		/// <summary>
		/// Enable the named filter for this current session.
		/// </summary>
		/// <param name="filterName">The name of the filter to be enabled.</param>
		/// <returns>The Filter instance representing the enabled fiter.</returns>
		public IFilter EnableFilter(string filterName) 
		{
			try
			{
				return _session.EnableFilter(filterName);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		/// <summary>
		/// Retrieve a currently enabled filter by name.
		/// </summary>
		/// <param name="filterName">The name of the filter to be retrieved.</param>
		/// <returns>The Filter instance representing the enabled fiter.</returns>
		public IFilter GetEnabledFilter(string filterName)
		{
			try
			{
				return _session.GetEnabledFilter(filterName);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		/// <summary>
		/// Disable the named filter for the current session.
		/// </summary>
		/// <param name="filterName">The name of the filter to be disabled.</param>
		public void DisableFilter(string filterName)
		{
			try
			{
				_session.DisableFilter(filterName);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		/// <summary>
		/// Create a multi query, a query that can send several
		/// queries to the server, and return all their results in a single
		/// call.
		/// </summary>
		/// <returns>
		/// An <see cref="IMultiQuery"/> that can return
		/// a list of all the results of all the queries.
		/// Note that each query result is itself usually a list.
		/// </returns>
		public IMultiQuery CreateMultiQuery()
		{
			try
			{
				return _session.CreateMultiQuery();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		/// <summary>
		/// Gets the session implementation.
		/// </summary>
		/// <remarks>
		/// This method is provided in order to get the <b>NHibernate</b> implementation of the session from wrapper implementions.
		/// Implementors of the <seealso cref="ISession"/> interface should return the NHibernate implementation of this method.
		/// </remarks>
		/// <returns>
		/// An NHibernate implementation of the <seealso cref="ISessionImplementor"/> interface 
		/// </returns>
		public NHibernate.Engine.ISessionImplementor GetSessionImplementation() 
		{
			try
			{
				return _session.GetSessionImplementation();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public FlushMode FlushMode {
			get {
				return _session.FlushMode;
			}
			
			set  {
				_session.FlushMode = value;
			}
		}
	}
}
