using System;
using System.Data;
using NHibernate;

namespace Orm
{
	public class PersistentTransaction
	{
		private ITransaction _transaction;
		private PersistentSession _persistentSession;

		public PersistentTransaction(ITransaction aTransaction, PersistentSession aPersistentSession)
		{
			_transaction = aTransaction;
			_persistentSession = aPersistentSession;
		}

		public void Begin() 
		{
			try
			{
				_transaction.Begin();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Begin(IsolationLevel isolationLevel)
		{
			try
			{
				_transaction.Begin(isolationLevel);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public void Commit()
		{
			_persistentSession.IsTransaction = false;
			try{
				_transaction.Commit();
			}catch(HibernateException e){
				throw new PersistentException(e);
			}
		}

		public override bool Equals(object obj)
		{
			return _transaction.Equals(obj);
		}
		
		public override int GetHashCode()
		{
			return _transaction.GetHashCode();
		}
		
		public void RollBack()
		{
			_persistentSession.IsTransaction = false;
			try
			{
				_transaction.Rollback();
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public bool IsActive 
		{
			get { return _transaction.IsActive; }
		}

		public override string ToString()
		{
			return _transaction.ToString();
		}
		
		public bool WasCommitted
		{
			get { return _transaction.WasCommitted; }
		}

		public bool WasRolledBack
		{
			get { return _transaction.WasRolledBack; }
		}

		public void Enlist(IDbCommand command)
		{
			try
			{
				_transaction.Enlist(command);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
	}
}
