using System;
using System.Runtime.CompilerServices;
using NHibernate.Cfg;
using NHibernate;
using Orm;

namespace Orm.Util
{
	public class AppBaseSessionController : SessionController
	{
		private ORMSessionFactory _sessionFactory = null;
		private PersistentSession _session = null;

		public AppBaseSessionController(Configuration cfg)
		{
			try
			{
				_sessionFactory = new ORMSessionFactory(cfg);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		#region SessionController Members
		[MethodImpl(MethodImplOptions.Synchronized)]
		public PersistentSession GetSession()
		{
			if(_session == null || !_session.IsOpen)
			{
				try
				{
					_session =  new PersistentSession(_sessionFactory.createSession(), _sessionFactory.Cfg);
				}
				catch(HibernateException e)
				{
					throw new PersistentException(e);
				}
			}
			return _session;
		}

		public void DisposeSession()
		{
			if(_session != null)
			{
				_session.Close();
				_session = null;
			}
		}

        public ORMInterceptor Interceptor
        {
            get
            {
                return _sessionFactory.Interceptor;
            }
        }

		#endregion
	}
}
