using System;
using System.Collections; 

namespace Orm.Util
{
	public class ORMCollection
	{
		private readonly object _owner;
		private readonly ORMAdapter _ownerAdapter;
		private readonly int _ownerKey;
		private readonly int _targetKey;
		private readonly bool _biDirectional;
		private readonly int _collType;

		public ORMCollection(object owner, ORMAdapter ownerAdapter, int ownerKey, int targetKey,
			bool biDirectional, int collType)
		{
			_owner = owner;
			_ownerAdapter = ownerAdapter;
			_ownerKey = ownerKey;
			_targetKey = targetKey;
			_biDirectional = biDirectional;
			_collType = collType;
		}

		protected ORMAdapter GetOwnerAdapter()
		{
			return _ownerAdapter;
		}

		protected IEnumerator GetEnumerator()
		{
			return _ownerAdapter.GetCollection(_ownerKey).GetEnumerator();
		}

		protected void Add(object target, ORMAdapter targetAdapter)
		{
			_ownerAdapter.GetCollection(_ownerAdapter.GetCollection(_ownerKey).Add(target));
			if(targetAdapter != null)
			{
				if(_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY)
				{
					targetAdapter.SetOwner(_owner, _targetKey);
				}
				else if(_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY)
				{
					IList list = targetAdapter.GetCollection(_targetKey);
					if(list != null) 
					{
						list.Add(_owner);
					}
				}
			}
		}

		public void Clear()
		{
			_ownerAdapter.GetCollection(_ownerKey).Clear();
		}

		public int Size()
		{
			return _ownerAdapter.GetCollection(_ownerKey).Count;
		}

		public bool IsEmpty()
		{
			return Size() == 0;
		}

		public void Remove(object target, ORMAdapter targetAdapter)
		{
			_ownerAdapter.GetCollection(_ownerKey).Remove(target);

			if(targetAdapter != null)
			{
				if(targetAdapter != null)
				{
					if(_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY)
					{
						targetAdapter.SetOwner(null, _targetKey);
					}
					else if(_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY)
					{
						IList list = targetAdapter.GetCollection(_targetKey);
						if(list != null)
						{
							list.Add(_owner);
						}
					}
				}
			}
		}

		protected bool Contains(object target)
		{
			return _ownerAdapter.GetCollection(_ownerKey).Contains(target);
		}

		protected void ToArray(object[] array)
		{
			_ownerAdapter.GetCollection(_ownerKey).CopyTo(array, 0);
		}

        internal IList GetCollection()
        {
            return _ownerAdapter.GetCollection(_ownerKey);
        }
	}
}
