using System;

namespace Orm.Util
{
	public class ORMList
	{
		private readonly object _owner;
	
		private readonly ORMAdapter _ownerAdapter;
	
		private readonly int _ownerKey;
	
		private readonly int _targetKey;
	
		private readonly bool _biDirectional;
	
		private readonly int _collType;
	
		public ORMList(object owner, ORMAdapter ownerAdapter, int ownerKey, int targetKey, bool biDirectional, int collType) 
		{
			this._owner = owner;
			this._ownerAdapter = ownerAdapter;
			this._ownerKey = ownerKey;
			this._targetKey = targetKey;
			this._biDirectional = biDirectional;
			this._collType = collType;
		}
	
		protected System.Collections.IEnumerator GetEnumerator() 
		{
			return _ownerAdapter.GetList(_ownerKey).GetEnumerator();
		}
	
		protected void Add(object target, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetList(_ownerKey).Add(target);
		
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{
					targetAdapter.SetOwner(_owner, _targetKey);
				}
				else if (_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY) 
				{
					System.Collections.IList list = targetAdapter.GetList(_targetKey);

                    if (list == null)
                    {
                        Iesi.Collections.ISet set = targetAdapter.GetSet(_targetKey);
                        if (set != null)
                        {
                            set.Add(_owner);
                        }
                    } 
                    else {
						list.Add(_owner);
					}
				}
			}
		}
	
		public void Clear() 
		{
			_ownerAdapter.GetList(_ownerKey).Clear();
		}
	
		public int Size() 
		{
			return _ownerAdapter.GetList(_ownerKey).Count;
		}
	
		public bool IsEmpty() 
		{
			return Size() == 0;
		}
	
		protected void Remove(object target, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetList(_ownerKey).Remove(target);
		
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{
					targetAdapter.SetOwner(null, _targetKey);
				}
				else if (_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY) 
				{
					System.Collections.IList list = targetAdapter.GetList(_targetKey);

                    if (list == null)
                    {
                        Iesi.Collections.ISet set = targetAdapter.GetSet(_targetKey);
                        if (set != null)
                        {
                            set.Remove(_owner);
                        }
                    }
                    else
                    {
						list.Remove(_owner);
					}
				}
			}
		}
	
		protected bool Contains(object target) 
		{
			return _ownerAdapter.GetList(_ownerKey).Contains(target);
		}
	
		protected void ToArray(System.Array a) 
		{
			_ownerAdapter.GetList(_ownerKey).CopyTo(a, 0);
		}
	
		protected object GetImpl(int index) 
		{
			return _ownerAdapter.GetList(_ownerKey)[index];
		}
	
		protected void RemoveImpl(int index) 
		{
			_ownerAdapter.GetList(_ownerKey).RemoveAt(index);
		}
	
		protected void InsertImpl(int index, object obj, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetList(_ownerKey).Insert(index, obj);
			
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{
					targetAdapter.SetOwner(_owner, _targetKey);
				}
				else if (_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY) 
				{
					System.Collections.IList list = targetAdapter.GetList(_targetKey);

                    if (list == null)
                    {
                        Iesi.Collections.ISet set = targetAdapter.GetSet(_targetKey);
                        if (set != null)
                        {
                            set.Add(_owner);
                        }
                    }
                    else
                    {
						list.Add(_owner);
					}
				}
			}
		}
	
		protected int IndexOf(object obj) 
		{
			return _ownerAdapter.GetList(_ownerKey).IndexOf(obj);
		}
	
		protected void SetImpl(int index, object obj) 
		{
			_ownerAdapter.GetList(_ownerKey)[index] = obj;
		}

        internal protected System.Collections.IList GetList()
        {
            return _ownerAdapter.GetList(_ownerKey);
        }
	}
}
