using System;

namespace Orm.Util
{
	public class ORMMap
	{
		private readonly object _owner;
	
		private readonly ORMAdapter _ownerAdapter;
	
		private readonly int _ownerKey;
	
		private readonly int _targetKey;
	
		private readonly bool _biDirectional;
	
		private readonly int _collType;
	
		public ORMMap(object owner, ORMAdapter ownerAdapter, int ownerKey, int targetKey, bool biDirectional, int collType) 
		{
			this._owner = owner;
			this._ownerAdapter = ownerAdapter;
			this._ownerKey = ownerKey;
			this._targetKey = targetKey;
			this._biDirectional = biDirectional;
			this._collType = collType;
		}
	
		protected System.Collections.IEnumerator GetEnumerator() 
		{
			return _ownerAdapter.GetMap(_ownerKey).Values.GetEnumerator();
		}
	
		protected void Add(object key, object target, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetMap(_ownerKey).Add(key, target);
		
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{
					targetAdapter.SetOwner(_owner, _targetKey);
				}
			}
		}
	
		public void Clear() 
		{
			_ownerAdapter.GetMap(_ownerKey).Clear();
		}
	
		public int Size() 
		{
			return _ownerAdapter.GetMap(_ownerKey).Count;
		}
	
		public bool IsEmpty() 
		{
			return Size() == 0;
		}
	
		protected void Remove(object target, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetMap(_ownerKey).Remove(target);
		
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{
					targetAdapter.SetOwner(null, _targetKey);
				}
			}
		}
	
		protected bool Contains(object target) 
		{
			return _ownerAdapter.GetMap(_ownerKey).Contains(target);
		}
	
		protected object GetImpl(object key) 
		{
			return _ownerAdapter.GetMap(_ownerKey)[key];
		}
	
		protected System.Collections.IEnumerator GetKeyEnumerator() 
		{
			return _ownerAdapter.GetMap(_ownerKey).Keys.GetEnumerator();
		}
	
		protected void ToArray(System.Array a) 
		{
			_ownerAdapter.GetMap(_ownerKey).Values.CopyTo(a, 0);
		}

        internal protected System.Collections.IDictionary GetMap()
        {
            return _ownerAdapter.GetMap(_ownerKey);
        }
	}
}
