using System;
using NHibernate;
using NHibernate.Cfg;

namespace Orm.Util
{
	public class ORMSessionFactory
	{
		private Configuration _cfg;
		private ISessionFactory _sessionFactory;
		private readonly ORMInterceptor _interceptor;

		public ORMSessionFactory(Configuration cfg)
		{
			_cfg = cfg;
			_sessionFactory = _cfg.BuildSessionFactory();
			_interceptor = new ORMInterceptor(_cfg);
		}

		public ISession createSession()
		{
			try
			{
				return _sessionFactory.OpenSession(_interceptor);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}

		public ISession createSession(System.Data.IDbConnection conn)
		{
			try
			{
				return _sessionFactory.OpenSession(conn, _interceptor);
			}
			catch(HibernateException e)
			{
				throw new PersistentException(e);
			}
		}
		
		public Configuration Cfg {
			get {
				return _cfg;
			}
		}

        public ORMInterceptor Interceptor
        {
            get
            {
                return _interceptor;
            }
            
        }
	}
}
