using System;

namespace Orm.Util
{
	public class ORMSet
	{	
		private readonly object _owner;
	
		private readonly ORMAdapter _ownerAdapter;
	
		private readonly int _ownerKey;
	
		private readonly int _targetKey;
	
		private readonly bool _biDirectional;
	
		private readonly int _collType;

		public ORMSet(object owner, ORMAdapter ownerAdapter, int ownerKey, int targetKey, bool biDirectional, int collType)
		{
			this._owner = owner;
			this._ownerAdapter = ownerAdapter;
			this._ownerKey = ownerKey;
			this._targetKey = targetKey;
			this._biDirectional = biDirectional;
			this._collType = collType;
		}

		protected System.Collections.IEnumerator GetEnumerator() 
		{
			return _ownerAdapter.GetSet(_ownerKey).GetEnumerator();
		}

		protected void Add(object target, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetSet(_ownerKey).Add(target);            
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{                    
					targetAdapter.SetOwner(_owner, _targetKey);
				}
				else if (_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY) 
				{
					Iesi.Collections.ISet set = targetAdapter.GetSet(_targetKey);

                    if (set == null)
                    {
                        Iesi.Collections.ISet list = targetAdapter.GetSet(_targetKey);
                        if (list != null)
                        {
                            list.Add(_owner);
                        }
                    }
                    else
                    {
                        set.Add(_owner);
					}
				}
			}
		}

		public void Clear() 
		{
			_ownerAdapter.GetSet(_ownerKey).Clear();
		}

		public int Size() 
		{
			return _ownerAdapter.GetSet(_ownerKey).Count;
		}
	
		public bool IsEmpty() 
		{
			return Size() == 0;
		}

		protected void Remove(object target, ORMAdapter targetAdapter) 
		{
			_ownerAdapter.GetSet(_ownerKey).Remove(target);
		
			if (targetAdapter != null) 
			{
				if (_collType == ORMBaseConstants.KEY_MUL_ONE_TO_MANY) 
				{
					targetAdapter.SetOwner(null, _targetKey);
				}
				else if (_collType == ORMBaseConstants.KEY_MUL_MANY_TO_MANY) 
				{
					Iesi.Collections.ISet set = targetAdapter.GetSet(_targetKey);

                    if (set == null)
                    {
                        Iesi.Collections.ISet list = targetAdapter.GetSet(_targetKey);
                        if (list != null)
                        {
                            list.Add(_owner);
                        }
                    }
                    else
                    {
						set.Remove(_owner);
					}
				}
			}
		}

		protected bool Contains(object target) 
		{
			return _ownerAdapter.GetSet(_ownerKey).Contains(target);
		}
	
		protected void ToArray(System.Array a) 
		{
			_ownerAdapter.GetSet(_ownerKey).CopyTo(a, 0);
		}

        internal Iesi.Collections.ISet GetSet()
        {
            return _ownerAdapter.GetSet(_ownerKey);
        }
	}
}
