using System;
using System.Threading;
using System.Collections;
using NHibernate;
using NHibernate.Cfg;

namespace Orm.Util
{
	public class ThreadBaseSessionController:SessionController
	{
		private ORMSessionFactory _sessionFactory = null;
		private Hashtable _sessions = null;
		public ThreadBaseSessionController(Configuration cfg)
		{
			try
			{
				_sessionFactory = new ORMSessionFactory(cfg);
				_sessions = new Hashtable();
			}
			catch(HibernateException e)
			{
				System.Console.WriteLine(" *** " + e.StackTrace);
				throw new PersistentException(e);
			}
		}
		#region SessionController Members

		public PersistentSession GetSession()
		{
			try
			{
				object obj = _sessions[Thread.CurrentThread];
				if(obj != null)
				{
					PersistentSession session = (PersistentSession)obj;
					if(!session.IsOpen)
					{
						session = new PersistentSession(_sessionFactory.createSession(), _sessionFactory.Cfg);
						_sessions[Thread.CurrentThread] = session;
					}
					return session;
				}
				else
				{
					PersistentSession session = new PersistentSession(_sessionFactory.createSession(), _sessionFactory.Cfg);
					_sessions[Thread.CurrentThread] = session;
					return session;
				}
			}
			catch(HibernateException)
			{
				throw new PersistentException("Failed to create PersistentSession");
			}
		}

		public void DisposeSession()
		{
			for (IEnumerator e = _sessions.Values.GetEnumerator(); e.MoveNext();) 
			{
				PersistentSession session = (PersistentSession) e.Current;
				if (session != null && session.IsOpen) 
				{
					session.Close();
				}
			}
			_sessions.Clear();
		}

        public ORMInterceptor Interceptor
        {
            get
            {
                return _sessionFactory.Interceptor;
            }
        }

		#endregion
	}
}
