using System;
using NHibernate;
using NHibernate.Cfg;
using System.Collections;

namespace Orm.Util
{
	public class UserDefinedSessionController:SessionController
	{
		private ORMSessionFactory _sessionFactory = null;
		private ArrayList _sessions;

		public UserDefinedSessionController(Configuration cfg)
		{
			try
			{
				_sessionFactory = new ORMSessionFactory(cfg);
				_sessions = new ArrayList();
			}
			catch(HibernateException)
			{
				throw new PersistentException("Failed to create ThreadBaseSessionController");
			}
		}
		#region SessionController Members

		public PersistentSession GetSession()
		{
			try
			{
				PersistentSession session = new PersistentSession(_sessionFactory.createSession(), _sessionFactory.Cfg);
				_sessions.Add(session);
				return session;
			}
			catch(HibernateException)
			{
				throw new PersistentException("Failed to create PersistentSession");
			}
		}

		public void DisposeSession()
		{
			for(IEnumerator e = _sessions.GetEnumerator(); e.MoveNext();)
			{
				PersistentSession session = (PersistentSession)e.Current;
				if(session != null || session.IsOpen)
					session.Close();
			}
			_sessions.Clear();
		}

        public ORMInterceptor Interceptor
        {
            get
            {
                return _sessionFactory.Interceptor;
            }
        }

		#endregion
	}
}
