/*
* Created on 2004/12/13
*
* DBTypeUtil.java
*/
namespace Ormtypemapper
{
	using System;
	/// <author>  Kit
	/// 
	/// </author>
	public class DBTypeUtil
	{
		public static DBType[] DBTypes
		{
			get
			{
				return DBTypeRepository.instance().DBTypes;
			}
			
		}
		/// <returns> DBType for integer, convenient for creating PK, index
		/// 
		/// </returns>
		public static DBType IntDBType
		{
			get
			{
				return getDBTypeByJava("int", null);
			}
			
		}
		
		public static DBType getDBTypeByID(int aID)
		{
			System.Collections.IDictionary lTypesMap = DBTypeRepository.instance().DBTypesMap;
			return (DBType) lTypesMap[aID];
		}
		public static DBType getDBTypeByJava(System.String aType, System.String aTypeModifier)
		{
			System.String lFullType = aType;
			// Only consider exceptional case: byte[]
			if (aType.Equals("byte") && aTypeModifier != null && aTypeModifier.Length > 0)
				lFullType += aTypeModifier;
			
			DBType[] lTypes = DBTypes;
			for (int i = 0; i < lTypes.Length; i++)
			{
				System.String lJavaType = lTypes[i].JavaType;
				if (lJavaType.ToUpper().Equals(lFullType.ToUpper()) || getJavaFullType(lJavaType).ToUpper().Equals(lFullType.ToUpper()))
				{
					return lTypes[i];
				}
			}
			for (int i = 0; i < lTypes.Length; i++)
			{
				if (lTypes[i].JavaWrapperType.ToUpper().Equals(lFullType.ToUpper()))
				{
					return lTypes[i];
				}
			}
			
			return null;
		}

		public static DBType getDBTypeByDB(int aDBType, System.String aType, int aLength)
		{
			DBType lPotentialMatch = null;
			DBType lAnyMatch = null; // Match length constraint = -1
			DBType lWorstMatch = null;
			
			aType = aType.Trim();
			
			// We renamed float in informix to informix_float
			if (aDBType == DBType.DBTYPE_INFORMIX)
			{
				if (aType.Equals("float"))
				{
					aType = "informix_float";
				}
				else if (aType.Equals("datetime"))
				{
					if (aLength == 1642)
					{
						aType = "datetime hour to second";
						aLength = 0;
					}
					else if (aLength == 4879)
					{
						aType = "datetime year to fraction";
						aLength = 5;
					}
				}
			}
			
			DBType[] lTypes = DBTypes;
			for (int i = 0; i < lTypes.Length; i++)
			{
				int[] lDBVendors = new int[]{aDBType, DBType.DBTYPE_GENERIC};
				for (int j = 0; j < lDBVendors.Length; j++)
				{
					int lDB = lDBVendors[j];
					System.String lType = lTypes[i].getDBType(lDB);
					System.String lTypeName = lTypes[i].getDBTypeName(lDB);
					if (lType.ToUpper().Equals(aType.ToUpper()) || lTypeName.ToUpper().Equals(aType.ToUpper()))
					{
						int lTypeLength = lTypes[i].getDBLength(lDB);
						if (lTypeLength == aLength)
						{
							// If exactly the same length then match
							return lTypes[i];
						}
						else if (lTypeLength == - 1)
						{
							if (lAnyMatch == null)
								lAnyMatch = lTypes[i];
						}
						else
						{
							if (lTypeLength > aLength)
							{
								// If length > required length, it is potentially matched 
								if (lPotentialMatch == null || lPotentialMatch.getDBLength(lDB) > lTypeLength)
								{
									// Update potentially matched type only if this type length is more closely to the required length
									lPotentialMatch = lTypes[i];
								}
							}
							else
							{
								// Length < required length
								if (lWorstMatch == null || lTypeLength > lWorstMatch.getDBLength(lDB))
								{
									lWorstMatch = lTypes[i];
								}
							}
						}
					}
				}
			}
			
			if (lPotentialMatch != null)
				return lPotentialMatch;
			
			if (lAnyMatch != null)
				return lAnyMatch;
			
			return lWorstMatch;
		}
		
		public static DBType getDBTypeBySQLType(int aSQLType)
		{
			DBType[] lTypes = DBTypes;
			for (int i = 0; i < lTypes.Length; i++)
			{
				if (lTypes[i].SqlType == aSQLType)
				{
					return lTypes[i];
				}
			}
			
			return null;
		}
		
		
		public static bool isIntType(int aTypeID)
		{
			DBType lDBType = getDBTypeByID(aTypeID);
			if (lDBType.TypeGroup == DBType.GROUP_LONG || lDBType.TypeGroup == DBType.GROUP_INTEGER || lDBType.TypeGroup == DBType.GROUP_SMALLINT || lDBType.TypeGroup == DBType.GROUP_TINYINT)
			{
				return true;
			}
			
			return false;
		}
		
		public static System.String getJavaFullType(System.String aJavaType)
		{
			return DBTypeRepository.instance().getJavaFullType(aJavaType);
		}
		
		//UPGRADE_NOTE: The initialization of  'JDBC_TYPE_NAME_MAP' was moved to static method 'ormtypemapper.DBTypeUtil'. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1005"'
		private static System.Collections.Hashtable JDBC_TYPE_NAME_MAP;
		
		public static System.String getJDBCTypeName(int aJDBCType)
		{
			return (System.String) JDBC_TYPE_NAME_MAP[aJDBCType];
		}
		static DBTypeUtil()
		{
			JDBC_TYPE_NAME_MAP = new System.Collections.Hashtable();
			{
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.BIT' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Boolean, "BIT");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.FLOAT' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Single, "FLOAT");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.DOUBLE' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Double, "DOUBLE");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.TINYINT' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.TinyInt, "TINYINT");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.SMALLINT' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.SmallInt, "SMALLINT");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.INTEGER' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Integer, "INTEGER");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.NUMERIC' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Numeric, "NUMERIC");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.BIGINT' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.BigInt, "BIGINT");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.DATE' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Date, "DATE");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.TIME' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.DBTime, "TIME");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.TIMESTAMP' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.DBTimeStamp, "TIMESTAMP");
//				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) 1, "BLOB");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.VARCHAR' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.VarChar, "VARCHAR");
//				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.CLOB, "CLOB");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.VARBINARY' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.VarBinary, "VARBINARY");
				//UPGRADE_TODO: The equivalent in .NET for field 'java.sql.Types.CHAR' may return a different value. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1043"'
				SupportClass.PutElement(JDBC_TYPE_NAME_MAP, (int) System.Data.OleDb.OleDbType.Char, "CHAR");
			}
		}
	}
}